#!/bin/bash

# Copyright (c) 2011 University of La Reunion - France
# Author:   Pascal ANELLI
# http://personnel.univ-reunion.fr/panelli
# All rights reserved.
#

# Goal
# Split a file organized in blocks  in several chunks
# Split when a blank line is occured
# 

#====================================================
# Procedures Definition
#====================================================

function usage () 
{
      echo 'Usage: sh $0 [intputfile]'
      echo "        [-h]              Print this help"
      echo "        [-v]              Verbose"
      echo "        [-c]              Clean up all tmp files"
      echo "        [-f]        output filename"
      echo "        inputfile   file to split"
      exit 1
}

#=======================================
##                 MAIN               ##
#=======================================

set -x

namescript=$(basename $0 .sh) ; # Erase the pathname
tmpdir="/tmp"
for dir in $(whoami) ${namescript}$$
do
       tmpdir=$tmpdir/$dir
       [ -d $tmpdir ] || mkdir $tmpdir
done

verbose=false
cleanup=false

while getopts "hvcf" option
do
case $option in
                h)
                        usage $*
                        ;;
                v)
                        verbose=true;
                        ;;
                c)
                        cleanup=true
                        ;;
                o|O)
                        opt="$OPTARG"
                        ;;
                f)
                        prefix=${OPTARG%.*}
                        suffix=${OPTARG##*.}
                        ;;
                \?)
                        echo "Error: Unknown option"
                        usage $*
        esac
done
shift `expr $OPTIND - 1`


if [ $# -eq 1 ] ; then
        arg=\$1
        prefix=${1%.*}
        suffix=${1##*.}
else
        arg="<&0"
fi


#echo $prefix
index=0
blockline="0 "`eval grep -n '^$' $arg | cut -f 1 -d :`
lastline=`eval "wc -l $arg" | awk '{print $1}'`
   # Add for computation in foreach loop
if [ `eval "tail -n 1 $arg" | wc -c` -ne 0 ]; then
                lastline=`expr $lastline + 1`
                blockline=${blockline}" "${lastline}
fi
echo $blockline 
   blocknum=0
   numbegin=0
   for i in $blockline ; do
                numend=$i
                #echo $numend
                if [ `expr $numend - $numbegin` -gt 1 ]; then
                        blocknum=`expr $blocknum + 1`             
                        tmpfile="${prefix}_${blocknum}.$suffix"
                        if [ ! -f $tmpfile ]; then
                                touch $tmpfile
                        fi
                        # delete the last empty line
                        lastline=`expr $numend - 1`
                        length=`expr $lastline - $numbegin` 
                        eval "head -n $lastline $arg" | tail -n $length >>$tmpfile
                fi
                numbegin=$numend
                #echo $numend
   done
exit 0
