#!/usr/bin/python
#
# Copyright (c) 2008 University of La Reunion - France
# Author:   Pascal ANELLI
# http://personnel.univ-reunion.fr/panelli
# All rights reserved.
# 
# Permission to use and copy this software in source and binary forms
# is hereby granted, provided that the above copyright notice, this
# paragraph and the following disclaimer are retained in any copies
# of any part of this software and that the University of La Reunion is
# acknowledged in all documentation pertaining to any such copy
# or derivative work. The name of the University of La Reunion may not
# be used to endorse or promote products derived from this software
# without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
# EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTIES OF 
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL 
# THE UNIVERSITY OF LA REUNION BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
# SOFTWARE.
#
#

#  filter packet trace file in order to keep flow with a beginning and a finishing 

#====================== Import section
import traceback, os
import tempfile
import string
from sys import stdout, stdin, stderr, exit
from optparse import OptionParser

#====================================================
# Constantes
#====================================================

#pkt trace file
#format
#    + 0.5 2 0 tcp 1250 ------- 0 2.0 3.0 0 0

FEVENT=0; FTIME=1; FINGRESS=2; FEGRESS=3; FPTYPE = 4
FPLEN=5; FFLAG=6; FFID=7; FSA=8; FDA=9; FSEQ=10; FUID=11

#====================================================
# Procedures Definition
#====================================================

def IsSender(segtype):
   isSource=False
   if not options.flow:
        if segtype == "ack":
           isSource= int(float(record[FDA])) == int(float(record[FEGRESS])) 
        elif segtype == "tcp":
           # print "tcp***"
           isSource= int(float(record[FSA])) == int(float(record[FINGRESS]))
   return (options.flow or ((not options.flow) and isSource))



#=======================================
##                 MAIN               ##
#=======================================
timetransit=0.0

parser = OptionParser(usage= 'usage: %prog [options] [trace file]')
parser.set_defaults(flow=False)

parser.add_option("-t", action='store', type='float', dest="timetransit",  help='time transit')
parser.add_option("-f", action='store_true', dest="flow", help='trace file is by flow (end to end)')
parser.add_option("-r", action='store', type='string', dest="filerej",    help='filename of rejected flow')
parser.add_option("-c", action='store_true', dest="cut", help='cut segment without ack at the end, transform flow in a completed flow')
parser.set_defaults(timetransit=0.0)

(options, args) = parser.parse_args()


if options.timetransit > 0.0:
        timetransit = options.timetransit
        

if len(args) == 0:
        fd = stdin
elif len(args) == 1:
        # Read the trace file
        filename=args[0]
        fd = open(filename, "r")
else:
	print "Usage: python %s [options] [filename]" % sys.argv[0]
	sys.exit(1)

temp, tempname=tempfile.mkstemp(suffix=".pkt", text=True)
fstate= {}
rejected=[]
transitionflow=0
#
# Phase 1: Erase flows STARTING in TRANSITION phase 
#
#  Make the list of correct FID (after transit time)
for line in fd:
       #if line.endswith("\n"):
       #                 line = line[:-1]
       if line == "\n":
                        os.write(temp, line)
                        continue
       if line.startswith('#'):
                        os.write(temp, line)
                        continue
       record=line.split()
       fid= int(record[FFID])
       # remove record in transition period
       if float(record[FTIME]) < timetransit:
       #          print "Rejected in transit period %d" % fid
                if  fid not in rejected:
                        rejected.append(fid)
                        transitionflow+=1
                continue
       # Now here we are after transition period
       # To find the completed flow: from seg 0 to last one
       # already seen this flow ?
       if fstate.has_key(record[FFID]):
                # Keep this line
                #print "keep ", line
                os.write(temp,line)
                
                # update seqno in flow state      
                seqno=int(record[FSEQ])
                if record[FPTYPE] == "tcp":
                        # are we on the source
                        if IsSender("tcp") and fstate[record[FFID]][0] < seqno:
                                #growth of the data seqno 
                                fstate[record[FFID]][0]=seqno
                else:
                        # keep ACK number if the number is greater
                        if IsSender("ack") and fstate[record[FFID]][1] < seqno:
                                fstate[record[FFID]][1]=seqno
                continue
       #new flow
       #print "new flow", line,
       if record[FPTYPE] =="tcp" :
         if IsSender("tcp"):
                #print "new flow"
                # keep only of those starting from seqno=0
                if record[FSEQ] == "0" and record[FEVENT] == "+":
                        state= [0, -1] # fid state: [seqno acknum]
                        fstate[record[FFID]]= state
                        #print "keep new fid: %s" % record[FFID]
                        os.write(temp, line)
                else:
                   #print "start with non zero seqno %d" % fid
                   if  fid not in rejected:
                        rejected.append(fid)            
fd.close()
os.close(temp)
#
# phase 2: Keep completed flow
#
#print fstate.keys()
if not options.cut:
        #print "no cut"
        # Keep real completed flow
        for fid in fstate.keys():
                state = fstate[fid]
                if state[0] != state[1]:
                        #print "------ INCOMPLET flow %s" % int(fid)
                        del fstate[fid]
                        rejected.append(int(fid))

        #print fstate.keys()
        #filter temp file
        #print tempname
        fd=file(tempname)
        for line in fd:
                #print "**" + line
                if line == "\n":
                      print line,
                      continue
                if line.startswith('#'):
                      print line,
                      continue
                record=line.split()
                if fstate.has_key(record[FFID]):
                        print line,
        os.unlink(tempname)
else:
    # keep all flow with the number of TCP data seg (seqno) <= last ACK num
    fd=file(tempname)
    for line in fd:
               #print line,
               if line == "\n":
                     print line,
                     continue
               if line.startswith('#'):
                     print line,
                     continue
               record=line.split()
               state=fstate[record[FFID]]
               if record[FPTYPE] == "tcp":
                      seqno=int(record[FSEQ])
                      if seqno > state[1]: # seqno > acknum
                                continue
               print line,
    os.unlink(tempname)
                 
if options.filerej != None :
        fd=open(options.filerej, "w")
        fd.write("# in transition period number of flow rejected: %d\n"%transitionflow)
        for fid in rejected:
                fd.write("%d\n"%fid)
        fd.close()

exit(0)
