#!/bin/bash
# print the flow characteristic for a specific flow (fid) list
# INPUT format: pkt file
# OUTPUT format 
#      1      2     3    4      5       6          7          8            9            10   
#     FFID, src, dst, START, Latency, Lastseqno, transmit, byte transmit, nb tcp drop, nb ack drop

#====================================================
# Procedures Definition
#====================================================

function usage () 
{
	echo "Usage: $0 TRACEFILE FID_list"
	exit 1
}


#===============================================================
#===                    AWK Procedure                        ===
#===============================================================

function doflowawk () {

flowawk="$tmpdir/flow.awk" 
cat >|$flowawk <<EOF
 BEGIN {
         FEVENT=1 ; FTIME=2; FINGRESS=3 ; FEGRESS=4
         FPTYPE=5 ; FPLEN=6; FFLAG=7 ; FFID=8
         FSA=9 ; FDA=10; FSEQ=11 ; FUID=12
	split(fid,tab)
        for (i in tab) {
		lfid[tab[i]]=i
		}
         }
! (\$FFID in lfid) { 
		next
	} 
/^\+/   { 
         if ((\$FINGRESS != int(\$FSA) ) || (\$FPTYPE != "tcp"))
                    next
         if (!(\$FFID in start) && (\$FSEQ == 0)) {
                lib[\$FFID] =(int(\$FSA) " " int(\$FDA))
                start[\$FFID] = \$FTIME; 
                lastseqno[\$FFID] = \$FSEQ
                drop[\$FFID]   = 0;
                dropa[\$FFID]   = 0;
                transmit[\$FFID] = 0;
                transmitb[\$FFID] = \$FPLEN;
          } else if (\$FFID in start) {
                transmitb[\$FFID] +=\$FPLEN
                if (\$FSEQ > lastseqno[\$FFID] ) {
                      lastseqno[\$FFID] = \$FSEQ
                } else {
                       transmit[\$FFID] +=1
                }                  
          }
        }
/^d/    {
         if (\$FFID in start) {
            if (\$FPTYPE == "tcp") {
                drop[\$FFID]  += 1;
            } else {
                dropa[\$FFID] += 1;
            }
         }
        }
/^r/    {
         if ((\$FEGRESS != int(\$FDA)) ||  (\$FPTYPE != "tcp") )
                  next
         if (lastseqno[\$FFID] == \$FSEQ) {
                             end[\$FFID] = \$FTIME
         }    
        }
END {
	print "#    fid lib  start, latency, size, transmit(pkt), transmit (byte), drop, drop ack"
        for (i in end) {
                if (lastseqno[i] == 0) { #recv SYN only
                    continue
                 }                 
                print i, lib[i], start[i], end[i]-start[i], lastseqno[i]+1, transmit[i], transmitb[i], drop[i], dropa[i];
        }
    }
EOF
}

#================================================================
export LC_ALL=C #For sort -n problem

#=======================================
##                 MAIN               ##
#=======================================
cleanup=false
allflow=false

while getopts "hcvfa" option
do
        case $option in
                h)
                        usage
                        ;;
                c)
                        cleanup=true
                        ;;
                v)
                        verbose=true
                        ;;
                a)   
                        allflow=true
                        ;;
		\?)
			echo "Unkown option $option" >&2
			usage
			;;
         esac
done
shift `expr $OPTIND - 1`


if [  $# -lt 1 ] ; then
	usage
fi
if [ $# -eq 2 ] ; then
	flowlist="$2"
fi
if $allflow ; then
	flowlist="$(fid_list.sh $1)"
fi

doflowawk
awk -v fid="$flowlist" -f $flowawk $1  | sort -k1g

if $cleanup ; then
        rm -rf $tmpdir
fi
exit 0

