#!/bin/bash
# print the pkt traces for a specific flow (fid)
#   

function usage () 
{
	echo "Usage: $0 [option] fid [pktfile] "
      	echo "Options:"
	echo "   -h              help"
	echo "   -c              erase tmpdir at the end"
	echo "   -v              invert-match: to select non-matching fid"
	echo "   -t <transit>    keep traces from this time"
	exit 1
}

#================================================================

#=======================================
##                 MAIN               ##
#=======================================
cleanup=false
inverse=0
timetransit=0.0
while getopts "hcvt:" option
do
        case $option in
                h)
                        usage
                        ;;
                c)
                        cleanup=true
                        ;;
                v)
                        inverse=1
                        ;;
                t)
                        timetransit="$OPTARG"
                        ;;
               \?)
                        echo "Unknown option $option" >&2
                        usage
                        ;;
         esac
done
shift `expr $OPTIND - 1`


if [  $# -lt 1 ] ; then
	usage
fi

if [[ $# -ge  2 && -f "$2" ]] ;
then
#        echo "file"
        arg=$2
else
#       echo "stdin"
        arg="<&0"
fi


namescript=$(basename $0 .sh) ; # Erase the pathname
tmpdir="/tmp"
for dir in $(whoami) ${namescript}$$
do
       tmpdir=$tmpdir/$dir
       [ -d $tmpdir ] || mkdir $tmpdir
done

tmpawk=${tmpdir}/grepfid.awk 
[ -e $tmpawk ] && rm -f $tmpawk
cat  >$tmpawk <<EOF
 BEGIN {
         FEVENT=1 ; FTIME=2
         FINGRESS=3 ; FEGRESS=4
         FPTYPE=5 ; FPLEN=6
         FFLAG=7 ; FFID=8
         FSA=9 ; FDA=10
         FSEQ=11 ; FUID=12
         }
\$FTIME < tt {
                next
        }
\$FFID == fid && inverse == 0 {
                print \$0
		next
         } 
\$FFID != fid && inverse == 1 {
                print \$0
	}
       
END {
    }
EOF

eval awk -f $tmpawk -v fid="$1" -v inverse=$inverse -v tt="$timetransit" $arg

if $cleanup ; then
        rm -rf $tmpdir
fi
exit 0



#awk  'BEGIN {split(fid,tab)
#                                 for (i in tab) {lfid[tab[i]]=i}}
#                         $8 in lfid { print $0 } '  
# | grep tcp | sort -k 11n -k2n 

#awk 'BEGIN {highseq=0;result=1} $5=="tcp" {if (($11== highseq) || ($11== highseq+1)) highseq= $11; else print $0}'

# grep tcp $1 | sort -n -k 8 -k 2 | awk '$1=="+" && $3==2 {print $0}' | awk 'BEGIN {highseq=0}; {if (($11>highseq) && ($11 > highseq+1)) {print $0}; highseq=$11}'
