#!/bin/bash

# use a pkt file
# Keep the traffic sending from node NodeId
# if node SA  <= Nodeid then keep the trace


function usage () 
{
      echo 'Usage: sh $0 [option] nodeid [pktfile]'
      echo "Options:"
      echo "   -h              help"
      echo "   -c              erase tmpdir at the end"
      echo "   -v              invert-match: to select node > node-id"
      echo "   -t  <time transit> time to ignore (<=)" 
      exit 1
}

function initdir () {
        namescript=$(basename $0 .sh) ; # Erase the pathname
        tmpdir="/tmp"
        for dir in $(whoami) ${namescript}$$
        do
                tmpdir=$tmpdir/$dir
                [ -d $tmpdir ] || mkdir $tmpdir
        done
}

function doawknode () {
awknode=${tmpdir}/$namescript.awk 
[ -e $awknode ] && rm -f $awknode
cat  >$awknode <<EOF
 BEGIN {
         FEVENT=1 ; FTIME=2
         FINGRESS=3 ; FEGRESS=4
         FPTYPE=5 ; FPLEN=6
         FFLAG=7 ; FFID=8
         FSA=9 ; FDA=10
         FSEQ=11 ; FUID=12
         }
\$FTIME < tt {
                next
             }
(int(\$FSA) <= nodeid || int(\$FDA) <= nodeid) && inverse == 0 {
         print \$0
         }
(int(\$FSA) > nodeid || int(\$FDA) > nodeid) && inverse == 1 {
         print \$0
         }
EOF
}


#================================================================
#=======================================
##                 MAIN               ##
#=======================================
cleanup=false
inverse=0
timetransit=0.0
while getopts "hcvt:" option
do
        case $option in
            h)
               usage
               ;;
            c)
               cleanup=true
               ;;
            v)
               inverse=1
               ;;
            t)
              timetransit="$OPTARG"
              ;;
            \?)
               echo "Unknown option $option" >&2
               usage
               ;;
         esac
done
shift `expr $OPTIND - 1`

#
# process arguments
#
if [ $# -lt 1 ] ; then
        usage
fi

if [[ $# -ge  2 && -f "$2" ]] ;
then 
#        echo "file"
        arg=$2
else
#       echo "stdin"
        arg="<&0"
fi

#
# Find out a tmp dir 
#
initdir

#    Build awk procedures
#------------------------
doawknode

#
# Do something
#
eval awk -f $awknode -v nodeid=$1 -v inverse=$inverse -v tt="$timetransit" $arg 
if [ $? -ne 0 ]; then
        echo "ERROR in $0" >&2
        exit 1
fi
#
# That's all folk
#
if $cleanup ; then
         rm -rf $tmpdir
fi
exit 0

