#!/bin/bash

# keep flow in one way and erase incomplete flow
# Called when the simulation has finished

#====================================================
# Procedures Definition
#====================================================

function usage () 
{
      echo "Usage: sh $0 [options] <tracefile> <onewaynode> <transit time>"
      echo "        [-h]              Print this help"
      echo "        [-v]              Verbose"
      echo "        [-c]              Clean up all tmp files"
      echo "        tracefile         pkt file"
      echo "        onewaynode        max id of node to keep"
      echo "        transit           transit time (simulation time to ignore)"
      exit 1
}


#================================================================
#=======================================
##                 MAIN               ##
#=======================================
namescript=${0%.sh};
namescript=${namescript##*/}; # erase the path

cleanup=false
opt=""
while getopts "hcvo:" option
do
        case $option in
                h)
                        usage
                        ;;
                v)
                        verbose=true;
                        ;;
                c)
                        cleanup=true
                        ;;
                o|O)   
                        opt="$OPTARG"
                        ;;
        esac
done
shift `expr $OPTIND - 1`

if [ $# -ne 3 ] ; then
        echo "ERROR: $0 wrong number of arg: $*" >&2
        usage >&2
        exit 1
fi

pktfile=$1
transit=$3
rootname=$(basename $pktfile .pkt); # erase the path
prefixpath=$(dirname $pktfile) ; # keep the path
TMPFILE=$(mktemp /tmp/${rootname}.XXXX)
rejname=$prefixpath/$rootname.rej
[ -e $rejname ] && rm -f $rejname

node_grep.sh $2 $pktfile | fid_completed.py -c  -r $rejname -t $transit >|$TMPFILE

if $verbose ; then
      if [ -s $rejname ] ; then
	   echo "Incompleted flows: $(cat $rejname | wc -l) flows"
      fi
fi
if $cleanup ; then
        rm -f $rejname 2>/dev/null
fi

if [ -s $TMPFILE ]; then
        mv -f $TMPFILE $pktfile
        exit 0
fi
exit 1


