#!/bin/bash

# Split a file in several chunk according to a period of time
# Input format:
#    time ....

#====================================================
# Procedures Definition
#====================================================

function usage () 
{
      echo "Usage: sh $0 [options] < tracefile> <transit time> <period> "
      echo "        [-h]              Print this help"
      echo "        [-v]              Verbose"
      echo "        [-c]              Clean up all tmp files"
      echo "        tracefile         pkt file"
      echo "        period            duration of the chunk "
      echo "        transit           transit time (simulation time to ignore)"
      exit 1
}


#================================================================
#=======================================
##                 MAIN               ##
#=======================================
namescript=${0%.sh};
namescript=${namescript##*/}; # erase the path

verbose=false
cleanup=false
opt=""
while getopts "hcvo:" option
do
        case $option in
                h)
                        usage
                        ;;
                v)
                        verbose=true;
                        ;;
                c)
                        cleanup=true
                        ;;
                o|O)   
                        opt="$OPTARG"
                        ;;
        esac
done
shift `expr $OPTIND - 1`

if [ $# -ne 3 ] ; then
        echo "ERROR: $0 wrong number of arg: $*" >&2
        usage >&2
        exit 1
fi

pktfile=$1
transit=$2
period=$3
rootname=${pktfile%.*}; # name of pktfile
prefixpath=$(dirname $pktfile) ; # keep the path

${0%.*}.py -t transit  $pktfile | e_blocksplit -f $rootname $2

exit 0


