#!/bin/bash

# input format: (one column)
#  int   (value)

# output
#    [x,] y

#====================================================
# Procedures Definition
#====================================================
function usage ()
{
      echo "Usage: $0 [options] [<data file>] "
      echo "   -h              help"
      echo "   -f              rank of input field"
      echo "   -b              bin size"
      echo "   -s              sample size"
      echo "   -c              1 number  2 pdf  3 cdf 4 ccdf"
      exit 1
}

#-----------------------------------------------------
source ~/0-Lib/lib/lib_stat.sh

#================================================================

#=======================================
##                 MAIN               ##
#=======================================

namescript=$(basename $0 .sh)
tmpdir="/tmp"
for dir in $(whoami) ${namescript}$$
do
       tmpdir=$tmpdir/$dir
       [ -d $tmpdir ] || mkdir $tmpdir
done


tmpfile=$tmpdir/$namescript.tmp

# column to use from t_pareto.tcl
#field=1 ; # Pareto
#field=2  ; # Pareto II

field=1
bin=1
case=""
nbrecord=0
while getopts hf:b:c:s: option
do
       case $option in
           h)
                usage
                ;;
           f)
                field=$OPTARG
                ;;
           b)
                bin=$OPTARG
                ;;
           c)
                case=$OPTARG
                ;;
           s)
                nbrecord=$OPTARG
                ;;
           \?)
                warg=`expr $OPTIND - 1`
                echo "Error: Unknown option ${!warg}" >&2
                usage
                ;;
        esac
done
shift `expr $OPTIND - 1`


if [ $# -eq 1 ]; 
then
        arg=\$1
else
        arg="<&0"
fi

{ eval awk \'NF == 0 \|\| /^\"/ \|\| /^#/ {next} {print \$$field}\' $arg ; } | sort -g  >|$tmpfile

if ((nbrecord == 0)) ; then
        nbrecord="$(wc -l <$tmpfile| tr -d ' ')"
        if ((nbrecord == 0)) ; then
                echo " Error: Empty file" >&2
                exit 1
        fi
fi
if [[ "$case" == "" || "$case" == 1 ]] ; then
        echo "# Number"
        awk -v binsize=$bin -f $roundawk $tmpfile | uniq -c |  awk -v binsize=$bin '{print $2*binsize,$1}' 
        #cat $tmpfile
        echo -e "\n"
fi
if [[ "$case" == "" || "$case" == 2 ]] ; then
        echo "# pdf"
        awk -v binsize=$bin -f $roundawk $tmpfile | uniq -c | tr -d '\r' | awk -f $pdfawk binsize=$bin
        echo -e "\n"
fi
if [[ "$case" == "" || "$case" == 3 ]] ; then
        echo "# cdf"
        # output of uniq: count value
        awk -v binsize=$bin -f $roundawk $tmpfile | uniq -c | tr -d '\r' | awk -f $cdfawk binsize=$bin sum=$nbrecord
        echo -e "\n"
fi

if [[ "$case" == "" || "$case" == 4 ]] ; then
        echo "# ccdf"
        # output of uniq: count value
        awk -v binsize=$bin -f $roundawk $tmpfile | uniq -c | tr -d '\r' | awk -f $ccdfawk binsize=$bin sum=$nbrecord
        echo -e "\n"
fi

rm -rf $tmpdir
exit 0
