#!/bin/bash
# Plot a datafile where 3blocks are made:
#       - repartition
#       - pdf
#       - cdf

#================================================================
export LC_ALL=C #For sort -n problem

#====================================================
# Constantes
#====================================================
#   "postscript eps enhanced color 'Helvetica' 12"
label=size
prefix=""
opt="set term aqua"

#====================================================
# Procedures Definition
#====================================================
function usage () 
{
      echo "Usage: $0 [options] [<data file>] "
      echo "Options:"
      echo "   -h              help"
      echo "   -t              terminal type:"
      echo "                            set term x11"
      echo "                            set term postscript eps enhanced color 'Helvetica' 12"
      echo "   -o              output file"
      echo "   -l              label for data"
      exit 1
}

#================================================================

#=======================================
##                 MAIN               ##
#=======================================
namescript=$(basename $0 .sh) ; # Erase the pathname
tmpdir="/tmp"
for dir in $(whoami) ${namescript}$$
do
       tmpdir=$tmpdir/$dir
       [ -d $tmpdir ] || mkdir $tmpdir
done
              
prefix=""
while getopts ht:T:o:l: option
do
       case $option in
           h)
                usage
                ;;
           t|T)
                opt=$OPTARG
		;;
           o)
                prefix=$OPTARG
                ;;
           l)
                label=$OPTARG
                ;;
           \?)
                echo "Error: Unknown option"
                usage $*
                exit 1
                ;;
        esac
done
shift `expr $OPTIND - 1`


if [ $# -eq 1 ]; then
	filename=$1
        prefix=${filename%.*}
else
        if  [ "$prefix" == "" ] ; then
        	filename=$(mktemp $tmpdir/datafile.XXXX)
                prefix=$(basename ${filename%.*})
        else
                filename=${prefix}
                prefix=${filename%.*}
        fi
        cat >| $filename <&0
fi	 


#echo  "# plot flow number, $label  (Flow ordered by $label)"
gnuplot -persist <<!
$opt
set output "${prefix}flow.eps"
set xlabel "Flow count"
set ylabel "$label"
unset key
unset title
set grid
set xtics border nomirror 
set mxtics 10
set ytics border nomirror  
set border 3
set logscale y
set title "Flow $label repartition"

plot '$filename' index 0 using (\$1) with lines
!

#echo "# Make de pdf of size"
gnuplot -persist <<!
$opt
set output "${prefix}pdf.eps"
unset key
set grid
set xtics border nomirror 0,100
set ytics border nomirror 
set border 3
set xlabel "$label"
set ylabel "Prob"
set title "Probability density function"
set xrange [0:1000]
set logscale y
set mxtics 5
set mytics 10
plot '$filename' index 1 using (\$1):(\$2) with lines
!


# for a graph F(size)= Prob[S < size]
#####################################
#echo "# Make de cdf of size"

gnuplot -persist <<!
$opt
set output "${prefix}cdf.eps"
set xlabel "$label"
set ylabel "Prob"
set grid
set style data lines
set mytics 2
set mxtics 10
set xtics border nomirror 
set ytics border nomirror 0,0.1
set yrange [0:1]
set logscale x
set border 3
unset key
set title "Cumulative data function"

plot '$filename' index 2 using (\$1):(\$2)  with lines 
!

rm -rf $tmpdir

exit 0
