#!/bin/bash
#
# Interface for generate a request in interactive mode to test-metric.tcl

#====================================================
# Procedures Definition
#====================================================

function usage () 
{
      echo "Usage: sh $0 [options]  file.pkt"
      echo "        [-h]              Print this help"
      echo "        [-c]              Clean up"
      echo "        [-v]              Verbose"
      echo "        [-q]              Quiet (don't plot the graph)"
      echo "        [-s scope]        scope of analysis"
      echo "        [-w who]          object id of this analysis"
      echo "        [-m metric]       metric to apply"
      echo "        [-p parameter]    parameter for the metric format '{name value}...'"
      exit 1
}

#====================================================
# Variables
#====================================================
# default value
cleanup=false
verbose=false

ext=""
scope=Flow
who=*
metric=""
parameter=""

#================================================================
export LC_ALL=C #For sort -n problem

#=======================================
##                 MAIN               ##
#=======================================

namescript=$(basename $0 .sh) ; # Erase the pathname
tmpdir="/tmp"
for dir in $(whoami) ${namescript}$$
do
       tmpdir=$tmpdir/$dir
       [ -d $tmpdir ] || mkdir $tmpdir
done


while getopts "hcvs:w:m:qp:" option
do
        case $option in
		h)
                        usage
                        ;;
		v)
			verbose=true;
		        ext="$ext -verbose 1 "
			;;
		c)
			cleanup=true
		        ext="$ext -cleanup 1"
			;;
		q)
		        ext="$ext -quiet 1"
			;;
 		s)
		        scope="$OPTARG"
			;;
 		w)
		        who="$OPTARG"
			;;
 		m)
		        metric="$OPTARG"
			;;
                p)
                        parameter="$OPTARG"
                        ;;
		\?)
			echo "Error: unknow option"
			exit 1
			;;
       esac
done
shift `expr $OPTIND - 1`

if [ $# -ne 1 ]; then
        echo "ERROR: Missing tracefile"
        usage 
fi
filename=$1
prefix="$(basename ${filename%.*})"
wd="$(dirname $filename)"

filerc=$tmpdir/${namescript}.rc
cat >|$filerc  << EOF
set conf(pkt)     { \
                    { $scope { \
                            { { $who } { $metric }} \
                          } \
                    } \
                  }
set conf(parameter) {$parameter}                  
EOF

ns $LIBSUITE/post.tcl -wd "$wd" -prefix $prefix -f $filerc $ext
exit $?
