#!/bin/bash
#  sum the lines filed by field in one line
#====================================================
# Procedures Definition
#====================================================

function usage () 
{
      echo "Usage: sh $0 [-v] [-l label] [<inputfile>]"
      exit 1
}

#================================================================
#====================================================
# Default Parameters
#====================================================

verbose=false
label="x"

#=======================================
##                 MAIN               ##
#=======================================

while getopts "l:vh" option
do
        case $option in
                h)
                        usage
                        ;;
                l)
                        label=$OPTARG
                        ;;
                v)      
                        verbose=true
                        ;;
               \?)
                        echo "Error: Unknown option"
                        usage
                ;;
        esac
done
shift `expr $OPTIND - 1`

if [ $# -eq 1 ] ; then
        arg=\$1
else
        arg="<&0"
fi


tmpdir=/tmp/$(whoami)
[ -d $tmpdir ] || mkdir $tmpdir
namescript=$(basename $0 .sh) ; # Erase the pathname
tmpdir=/tmp/$(whoami)/${namescript}$$
[ -d $tmpdir ] || mkdir $tmpdir


tmpawk=$tmpdir/sum.awk 
cat >|$tmpawk <<EOF
BEGIN {
        maxf=0
}
/#/ || /^$/ {
                next
       }
NF > 1 { if (NF > maxf)
                maxf = NF;
         for (i = 2; i <= NF; i++) {
                if (i in sum) {
                        sum[i] += \$i;
                } else {
                        sum[i] = \$i;
                }
          }
        }
END {
        printf "%s ", confid
        for (i=2; i <= maxf; i++) {
                printf "%g ", sum[i]
        }
        print ""
}
EOF
eval awk -f $tmpawk confid="$label" $arg

rm -rf $tmpdir
exit 0
