#!/bin/bash

# Copyright (c) 2012 University of La Reunion - France
# Author:   Pascal ANELLI
# http://personnel.univ-reunion.fr/panelli
# All rights reserved.
#


# procedure to analyze data

set -u ; # unset variables as an error


#===============================================================
#===                    AWK Procedure                        ===
#===============================================================

#-------------------------------------------------
#  weighted Average per x
# input
#     x  coly    colyd
#        numerator   denominator
# output
#        x   average
function dowavgawk () {
wavgawk=$tmpdir/wavg.awk
cat >| $wavgawk <<EOF
BEGIN {
        x=-1
        weight=0
        sum=0.0
        colyd=0
}
function pline() {
        if (weight != 0) {
                print x, sum*1.0/weight 
        }
}
\$1~/#/ {next}
x != \$colx {
        pline()
        sum=0.0;
        weight=0 
        x=\$colx
     }
{
     sum+=\$coly;
     if (colyd ==0) {
                weight +=1
     } else {
                weight +=\$colyd
     }
}
END {
     pline()
}
EOF
}

#
# Sustainable  Weighted average per x
# input
#     x   coly  field+2
#         value  weight(denominator)
# output
#        x   average
function dowavgsusawk () {
wavgsusawk=$tmpdir/wavgsus.awk
cat >| $wavgsusawk <<EOF
BEGIN {
        x=-1
        weight=0
        sum=0.0
        colyd=0
}
function pline() {
        if (weight != 0) {
                print x, sum*1.0/weight
        }
}
\$1~/#/ {next}
x != \$colx {
        pline()
        x=\$colx
     }
     {
      sum+=\$coly
      if (colyd ==0) {
                weight +=1
        } else {
                weight +=\$colyd
       }
     }
END {
        pline()
}
EOF
}


#
# Weighted floating Average per x
function dowavgfawk () {
wavgfawk=$tmpdir/wavgf.awk
cat >| $wavgfawk <<EOF
BEGIN {
        x=-1 ; colyd=0
        sum=0.0; sumx=0.0
        weight=0.0; weightx=0.0
        for (i=0; i<= maxslot ; i++) {
                num[i]=0.0; denum[i]=0.0
        }
        nbx=0

}
function pline() {
             nbx++
             i=nbx%maxslot
             sum-=num[i]; num[i]=sumx
             weight-=denum[i]; denum[i]=weightx
             sum+=sumx
             weight+=weightx

             if (weight > 0 )
                printf "%d %-8.5f\n", x, sum/weight
     }
\$1~/#/ {next}
\$colx != x {
                pline()
                x=\$colx
                sumx=0.0; weightx=0.0
             }   
\$coly ~ /[0-9.\-]/ { 
          if (colyd ==0) {
                  weightx +=1
          } else {
                  weightx +=\$colyd
          }
          sumx+=\$coly
         }
END {
        pline()
}

EOF
}



# Average per x
# output format:
#    x, mean, min, max, stdev, nbsample 
function doavgawk () {
avgawk=$tmpdir/avg.awk
cat >| $avgawk <<EOF
BEGIN {
        x=-1
        nbsample=0
}
function pline () {
        if (nbsample!= 0) {
                mean=sum1*1.0 / nbsample
                variance = (sumsquare*1.0/ nbsample) - (mean*mean)
                if (variance < 0) 
                          variance=0.0
                stdev = sqrt(variance)
                print x, mean, min, max, stdev, nbsample
        }
}
\$1~/#/ {next}
x != \$colx {
       pline()
       sum1=0.0
       sumsquare=0.0
       nbsample=0
       max=0
       min=-1
       x=\$colx
     }
     {
      nbsample +=1
      sumsquare+= \$coly * \$coly 
      sum1+=\$coly
      if (\$coly > max)
                  max=\$coly
      if ((\$coly < min) || (min == -1))
                  min=\$coly
     }
END {
     pline()
    }
EOF
}


#------------------------------------------------------
#
# Sum per x
#
function dosumawk () {
sumawk=$tmpdir/sum.awk
cat >| $sumawk <<EOF
BEGIN {
        x=-1
}
function pline () {
        if (x >= 0) {
                print x, sum, sumd, nbrecord
        }
}

\$1~/#/ {next}
x != \$colx {
             pline()
             sum=0
             sumd=0
             nbrecord=0
             x=\$colx
           }
          {
           sum+=\$coly
           nbrecord+=1
           if (colyd == 0)
                sumd+= 1
           else 
                sumd+=\$colyd
          }
 END {
        pline()
 }
EOF
}


#------------------------------------------------------
#
# Sum 
#
function dosummawk () {
summawk=$tmpdir/summ.awk
cat >| $summawk <<EOF
BEGIN {
        sum=0
}
\$1~/#/ {next}
{
	first = \$1;
	sum = sum + \$2;
	print first, \$2, sum;
}
EOF
}



#---------------------------------------------------------------
#	Smoothing the curves

#  Floating average
function doavgfawk () {
avgfawk=$tmpdir/avgf.awk
cat >| $avgfawk <<EOF
BEGIN {
        sum = 0.0
        nbrecord = 0
        for (i=0; i<= maxslot ; i++) {
                val[i]=0.0
        }
       }
\$1 ~ /#/ {next}
\$coly ~ /[0-9.\-]/{
        nbrecord++
        i=nbrecord%maxslot
        sum-=val[i]
        val[i]=\$coly
        sum += \$coly
        x= \$colx
	if (maxslot > nbrecord)
		div=nbrecord
	else
		div=maxslot
        print x, sum/div
       }
EOF
}


#
#  EWMA
function doavgeawk () {
avgeawk=$tmpdir/avge.awk
cat >| $avgeawk <<EOF
BEGIN {
        avg = 0.0
       }
\$1 ~ /#/ {next}
\$coly ~ /[0-9.\-]/{
	if (avg == 0) {
		avg=\$coly
        }
        avg *= (1-alpha)
        avg += \$coly * alpha
        print \$colx, avg
       }
EOF
}


#---------------------------------------------------------------
#
#  median per x
function domedianawk () {
medianawk=$tmpdir/median.awk
cat >| $medianawk <<EOF
BEGIN {
        x=-1
}
function sort(tab,numel,rtab) {
    for (j=1 ; j<= numel; j++) {
        max=-1
        for (i in tab) {
                if (max <= tab[i]) {
                        max=tab[i]
                        item=i
                }
        }
        rtab[j]=max
        delete tab[item]
    }
    for (i in tab) 
        delete tab[i] ;

}
function pline() {
         if (x == -1) 
                return
         middle=int((record+1)/2.0)
         sort(tab,middle+1,rtab)
         odd=(record/2.0 - int(record/2.0) <= 0)
         if (odd) {
                median=(rtab[middle]+rtab[middle+1])/2.0
         } else {
                median=rtab[middle]
         }
         print x, median
}
\$1~/#/ {next}
\$colx != x {
             pline()
             x=\$colx
             record=0
          }           
         {
         record +=1
         tab[record]=\$coly
         }
END {
      pline()
}

EOF
}

#----------------------------------------------------------------------
#
#  proportional average per x
function doavgpawk () {
avgpawk=$tmpdir/avgp.awk
cat >| $avgpawk <<EOF
BEGIN {
        x=-1
        max=0
}
\$1~/#/ {next}
        {if (\$colx >= max) {
                if (x != -1) {
                        print x, sum*1.0/nbrecord
                }
                x=\$colx
                max=\$colx*(prop+1.0)
                fraction=max- int(max)
                if (fraction > 0)
                      max=int(max)+1
                sum=0.0
                nbrecord=0
          }           
         nbrecord +=1
         sum+=\$coly
         }
END {
        if (nbrecord!= 0) {
                print x, sum*1.0/nbrecord
        }
}

EOF
}


#
# Weighted Average acccording to a proportional window per x
function dowavgpawk () {
wavgpawk=$tmpdir/wavgp.awk
cat >| $wavgpawk <<EOF
BEGIN {
        x=-1
        max=0
}
function pline() {
        if (x != -1) {
                print x, sum*1.0/nbrecord
        }
}
\$1~/#/ {next}
        {if (\$colx >= max) {
                pline()
                x=\$colx
                max=\$colx*(prop+1.0)
                fraction=max- int(max)
                if (fraction > 0)
                      max=int(max)+1
                sum=0.0
                nbrecord=0
          }           
         nbrecord +=\$colyd
         sum+=\$coly
         }
END {
	pline()
}
EOF
}



#---------------------------------------------------------------
#
#  difference prob per x
function dodifference () {
difference=$tmpdir/difference.awk
cat >| $difference <<EOF
BEGIN {x=-1}
function pline() {
        if (x == -1)    
                return
        #       drop-favor neg, pos    #      #         droptail     total
        print x, record, sumn, sump, negatif, positif, sumdn, sumdp, sumd
}
\$1~/#/ {next}
\$colx != x {
             pline()
             x=\$colx
             record=0
             positif=0
             negatif=0
             sump=0
             sumn=0
             sumdn=0.0
             sumdp=0.0
             sumd=0.0
          }
        {
         record +=1
         sumd+=\$coly
         y=int(1000*(\$coly-\$colyb))
         if (y >0) {
                positif+=1
                sump+=\$coly-\$colyb
                sumdp+=\$coly
         } else  if (y <0) {
                negatif+=1
                sumn+=\$coly-\$colyb
                sumdn+=\$coly
                }
        }
END{
        pline()
}
EOF
}




#---------------------------------------------------------------------------
# Normalized thrughput
# Output format
#  size  thruput_dt

function dosthruawk () {
sthruawk=$tmpdir/sthru.awk
cat >| $sthruawk <<EOF
BEGIN {
        bw=bw+0
        sum=0.0
        delay=0.0
        x=-1
}
  function pline() {
        if (delay > 0 )
                printf "%d %6.2f\n", x, sum/delay
  }
\$1~/#/ {next}
\$colx != x {
              pline()
              x=\$colx
            }
            {
              thru=(\$(coly+4)*800.0/bw);
              sum+=thru
              delay+=\$coly
            }
END  {  
      pline()
     }
EOF
}

function dogoodputsawk () {
goodputsawk=$tmpdir/goodputs.awk
cat >| $goodputsawk <<EOF
BEGIN {
        bw=bw+0
        sum=0.0
        delay=0.0
        x=-1
}
  function pline() {
        if (delay > 0 )
                printf "%d %6.2f\n", x, sum/delay
  }
\$1~/#/ {next}
\$colx != x {
              pline()
              x=\$colx
            }
            {
              thru=(((\$2-1)*1500+40)*800.0/bw);
              sum+=thru
              delay+=\$coly
            }
END  {  
      pline()
     }
EOF
}


function dofthruawk () {
fthruawk=$tmpdir/fthru.awk
cat >| $fthruawk <<EOF
BEGIN {
        bw=bw+0
        sum=0.0; sumx=0.0
        delay=0.0; delayx=0.0
        x=-1
        for (i=0; i<= maxslot ; i++) {
                num[i]=0.0; denum[i]=0.0
        }
        nbx=0
}
function pline() {
             nbx++
             i=nbx%maxslot
             sum-=num[i]; num[i]=sumx
             delay-=denum[i]; denum[i]=delayx
             sum+=sumx
             delay+=delayx

             if (delay > 0 )
                printf "%d %6.2f\n", x, sum/delay
     }
\$1~/#/ { next }
\$colx != x {
             pline()
             x=\$colx
             sumx=0.0; delayx=0.0
            }
            {
              thru=(\$(coly+4)*800.0/bw);
              sumx+=thru
              delayx+=\$coly
            }
END  {  
        pline()
     }
EOF
}

#------------------------------------------
#     Stats
#------------------------------------------

function doround ()
{
roundawk=$tmpdir/round.awk
[ -e $roundawk ] && rm -f $roundawk
cat >$roundawk <<EOF
BEGIN {
        init= -1
}
{
	if (\$1 < 0 ) {
                quot=int(\$1/binsize)
                if (int(quot*binsize*100000) ==  int(\$1*100000))
		        round=\$1
                else
                        round=\$1-binsize
	} else {
		frac=\$1%binsize
                if (frac > 0.0)
                        round=(\$1+binsize)
                else
                        round=\$1
             }
	bin = int(round/binsize)
        printf "%d\n", bin
}
END {
}
EOF
}

function dopdf () {
pdfawk=$tmpdir/pdf.awk
cat >| $pdfawk <<EOF
BEGIN {
        FOCCUR=1
	FBIN=2 ; 
	nsample=0
	maxbin=-1
      }
      {
	if (\$FBIN> maxbin ) {
		maxbin = \$FBIN
	}
	tab[\$FBIN] += \$FOCCUR
	nbsample+=\$FOCCUR
      }	
END {
        if (nbsample > 0) {
	    for (j=0 ; j<=maxbin ; j++) {
		#       x,       pdf
		printf "%-8.5f %-8.6f\n", j*binsize, 1.0*tab[j]/nbsample 
	  }
       }
    }
EOF
}

# use an array
# write one line by value [0 to maxbin]
function docdf2 ()
{
cdf2awk=$tmpdir/cdf2.awk
cat >| $cdf2awk <<EOF
BEGIN {
        FOCCUR=1
	FBIN=2 ; 
	nsample=0
	maxbin=-1
}
      {
	if (\$FBIN> maxbin ) {
		maxbin = \$FBIN
	}
	tab[\$FBIN] += \$FOCCUR
	nbsample+=\$FOCCUR
      }	
END {
        if (nbsample > 0) {
	    sum=0.0
	    for (j=1 ; j<=maxbin ; j++) {
		sum+=tab[j]
		#  x, cdf
		printf "%-8.5f %-9.6f\n", j*binsize, 1.0*sum/nbsample 
	    }
       }
}
EOF
}

# need sum in argument
# print a line on each line read
function docdf ()
{
cdfawk=$tmpdir/cdf.awk
[ -e $cdfawk ] && rm -f $cdfawk
cat >$cdfawk <<EOF
BEGIN {
        FOCCUR=1
	FBIN=2 ; 
        init= -1
	nsample=0
	maxbin=-1
}
{      if (init) {
                init=0
                bin=\$FBIN
                nbrecord=\$FOCCUR
                next
        }
       if (bin != \$FBIN) {
               printf "%-8.5f %-9.7f\n", bin*binsize, nbrecord*1.0/sum
	       bin=\$FBIN
        } 
        nbrecord +=\$FOCCUR
}
END {
     printf "%-8.5f %-9.7f\n", bin*binsize, nbrecord*1.0/sum        
}
EOF
}


function dopercentile ()
{
cdfawk=$tmpdir/cdf.awk
[ -e $cdfawk ] && rm -f $cdfawk
cat >$cdfawk <<EOF
BEGIN {
        FVAL=1
        init= -1
	for (j=1 ; j<=100 ; j++) {
		tab[j]=0
	}
}
{      if (init) {
                binsize=int(maxbin-\$FVAL)/100
		init=0
       }
       frac=\$FVAL%binsize
       if (frac > 0.0)
                        round=(\$FVAL +binsize)
                else
                        round=\$FVAL
       	}
       bin = int(round/binsize)
       tab[bin] += 1
       nbsample+= 1
}
END {
	if (nbsample > 0) {
	    for (j=0 ; j<=maxbin ; j++) {
		#       x,       pdf
		printf "%-8.5f %-9.7f\n", j, 1.0*tab[j]/nbsample 
	  }
       }
}
EOF
}


function doccdf ()
{
ccdfawk=$tmpdir/ccdf.awk
[ -e $ccdfawk ] && rm -f $ccdfawk
cat >$ccdfawk <<EOF
BEGIN {
        FOCCUR=1
	FBIN=2 ; 
        init= -1
}
{      if (init) {
                init=0
                bin=\$FBIN
                nbrecord=\$FOCCUR
                next
        }
       if (bin != \$FBIN) {
               printf "%-8.5f %-9.7f\n", bin*binsize, 1-(nbrecord*1.0/sum)
	       bin=\$FBIN
        } 
        nbrecord +=\$FOCCUR
}
END {
     printf "%-8.5f %-9.7f\n", bin*binsize, 1-nbrecord*1.0/sum        
}
EOF
}




#================================================================

namescript=$(basename $0 .sh) ; # Erase the pathname
tmpdir="/tmp"
for dir in $(whoami) ${namescript}$$
do
       tmpdir=$tmpdir/$dir
       [ -d $tmpdir ] || mkdir $tmpdir
done


#    Build awk procedures
#------------------------
dowavgsusawk
dowavgawk
dowavgfawk

dosumawk
dosummawk
doavgawk
doavgpawk
domedianawk 

doavgfawk
doavgeawk

dodifference

dogoodputsawk
dosthruawk

doround
docdf
doccdf
dopdf
docdf2
