#!/usr/bin/otclsh
#
# Copyright (c) 2008 University of La Reunion - France
# Author:   Pascal ANELLI
# http://personnel.univ-reunion.fr/panelli
# All rights reserved.
# 

# load common library
set currentdir [pwd]
catch "cd $env(LIBSUITE)"
source analy-metric.tcl
catch "cd $currentdir"


#ns ns-math.tcl -ifile test_1_xx.rtt -opref test

# Call mathematics functions applied on a file structured in block
#each line is a replication, each column is a parameter
# each block is separated by 2 blank lines

#========================
# set default parameters
#========================
set param(ifile)        {} ; # inputfile
set param(opref)        {} ; # output prefix  
set param(metrics)      {} ; # What to do
set param(parameters)   {{weight 3} {accuracy 1.0/100}}; #
set param(xy)          {1 2}; # Which column to use 

#======================
#      PROCEDURE
#======================

# create a Analysis object for each block
proc CreateAnalysis {inputfile todo parameters xy} {

#set nbdata [expr [exec grep -c {^$} $inputfile] /2 - 1]
set awk_code {
        BEGIN {
               blocnum = 0;
               found = 0;
               blankline=2
        }
        { if (!($0~/^$/)) {
                # new data line
                if (!found && blankline >= 2) {
                        ++blocnum
                        found= 1
                        blankline = 0
                }
          } else {
                # empty line
                ++blankline
                found =0
          }
        }
END {
      print blocnum
   }
}

set lanalysis {}
# number of block
set nbdata [exec awk $awk_code $inputfile]
#create an analysis object per block
for {set i 0 } {$i < $nbdata} {incr i} {
        #puts stderr "Create block $i with $todo"
        lappend lanalysis [new Analysis $inputfile $i $todo $parameters $xy] 
}
return $lanalysis
}


#  call the processing method to apply metrics on each Analysis object
proc ComputeData {lmet} {

     set ldatafile {}
     foreach met $lmet {
        #puts stderr "Apply metrics for [lsearch $lmet $met]"
        set ldatafile [concat $ldatafile [$met MakeData]]
     }
     return $ldatafile
}


# concat the result files according to the processing done
proc DoOneFile {filelist opref todo} {

set log {}
foreach mathf $todo {
        if { $opref == "" } {
                set savefile /dev/stdout
        } else {
                set savefile [makefile [format "%s%s" $opref $mathf] [GetSuffix data]]
                puts stderr "for $mathf result file is: $savefile"
       }
     foreach item $filelist {
            if {[lindex $item 0] == "$mathf" } {
               exec cat [lindex $item 1] >>$savefile
               exec echo "\n" >>$savefile
#              exec rm -f [lindex $item 1]
            }
     }
}
}


#======================
#   MAIN
#======================


global argc argv argv0
global param

#  Read the command line
if {([expr $argc % 2])} {
                puts "num: $argc"
                puts "     $argv"
                usage $argv0 [array names param]
}

# Store line arguments in array param
for {set i 0} {$i < $argc} {incr i} {
                set arg [lindex $argv $i]
                if {[string range $arg 0 0] != "-"} continue

                set name [string tolower [string range $arg 1 end]]
                set param($name) [lindex $argv [expr $i+1]]
}

# Read the ressource file
if { [info exists param(f)] } {
        source $param(f)
}

# Call to create a Analysis object (in order to call the routines of metrics list)
set lanalysis [CreateAnalysis  \
                        $param(ifile) \
                        $param(metrics) \
                        $param(parameters) \
                        $param(xy)]
# Applied metrics on each block
set lresult [ComputeData $lanalysis]
#puts stderr "log file: $lresult"
# Concate the result for each block in one file
DoOneFile $lresult $param(opref) $param(metrics)
#  TO DO clear input file

