#
# Copyright (c) 2008 The University of La Reunion - France
# Authors:   P. ANELLI
#
# http://personnel.univ-reunion.fr/panelli
# All rights reserved.
# 
# Permission to use and copy this software in source and binary forms
# is hereby granted, provided that the above copyright notice, this
# paragraph and the following disclaimer are retained in any copies
# of any part of this software and that the University of La Reunion is
# acknowledged in all documentation pertaining to any such copy
# or derivative work. The name of the University of La Reunion may not
# be used to endorse or promote products derived from this software
# without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
# EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTIES OF 
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL 
# THE UNIVERSITY OF LA REUNION BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
# SOFTWARE.
#
#

# create an error model according to a list given in parameter

#puts "#Load error model"

Class ErrorModel/Rate -superclass ErrorModel

Class Error 

        # link-lossmodel: Insert error model in the link 
        #                       (after the queue and link (delay))
        #                 Can be seem by nam (link-lossmodel)
        #                   call SimpleLink  insert-linkloss (bugged in ns)
        # lossmodel: Insert loss model at the entry before the queue 
        #               call SimpleLink errormodule 


Error instproc init {errorlist replication  droptarget {ecn False}} {
	$self instvar replication_ head_ tail_ serial_

	#puts "Error class: $error"
	set replication_ $replication
        set head_ {}
        set serial_ {}


        foreach error $errorlist {
                # Load tab array
                foreach param $error {
                        set name [lindex $param 0]
                        set tab($name) [lindex $param 1]
                }
                #Create ErrorModel
	        set emod [new ErrorModel/$tab(type)]
                # Configure errormodel
                $emod set markecn_ $ecn
                $emod set delay_pkt_ false

                if {$tab(type) == "List"} {
                        if { [info exists tab(actionlist)] } {
                                $emod droplist $tab(actionlist)
                           }
                }
                if { [info exists tab(paramlist)] } {
	                instance-parameter $emod $tab(paramlist)
                } 
                
	        if {[info exists tab(todo)]} {
	          if {$tab(todo) == "delay"} {
                         $emod set delay_pkt_ true
	  	         if {[info exists tab(rvdelay)]} {
                                #  rvdelay: "type name" "value"
		                $self add-rvdelay $emod \
                                                  [lindex $tab(rvdelay) 0] \
                                                  [lindex $tab(rvdelay) 1]
                         }
	          } elseif {$tab(todo) != "drop"} {
                        puts "WARNING: errormodel todo unkown" 
                 }
	        } else {
		   set tab(todo) "drop"
                }
                set tab(emod)  $emod
                # log the detail of this errormodel
                lappend serial_ [array get tab]
                unset tab
                
	        #create RNG 
                set randgen [new RNG]
                for {set i 1} {$i < $replication} {incr i} {
                        $randgen next-substream
                }
                set randvar [new RandomVariable/Uniform]
                $randvar use-rng $randgen
                $emod ranvar $randvar

                # connect emod
                $emod drop-target $droptarget
                if {$head_ == {}} {
                        set head_ $emod
                } else {
        	       $prev target $emod
                }
                set prev $emod
        }
        set tail_ $emod
}


Error instproc add-rvdelay {emod type paramlist} {
	$self instvar replication_

     #puts "  Add RV $type for delay_pkt: $paramlist"
     set randgen [new RNG]
     for {set i 1} {$i < $replication_} {incr i} {
              $randgen next-substream
     }
     set randomvar [new RandomVariable/$type]
     $randomvar use-rng $randgen
     instance-parameter $randomvar $paramlist
       	     
     $emod rvdelay $randomvar
}



Error instproc target {nxconnector} {
        $self instvar tail_

        $tail_ target $nxconnector
}


Error instproc drop-target {dropconnector} {
        $self instvar tail_

        $tail_ drop-target $dropconnector
}


Error instproc add-target {emodule} {
        $self instvar tail_

        $emodule insert $tail_
}


Error instproc get-head {} {
        $self instvar head_

        return $head_
}


Error instproc print {fd} {
        $self instvar serial_
        
        set i 1
        foreach item $serial_ {
                array set tab $item
                puts $fd "\t ErrorModel $tab(type) in position $i"
                foreach name [array names tab] {
                   puts $fd " \t\t\t$name: $tab($name)"
                }
                unset tab
                incr i
        }
}




Error instproc tracevar {todo filefd} {
	$self instvar serial_

        foreach item $serial_ {
              array set tab $item
              #puts ">> $tab(todo) $todo"
              if {$tab(todo) == $todo} {
                 # BaseTrace: to write a string in channel
	         set bt [new BaseTrace/Event]
                 #puts " attach filefd on $todo"
                 $bt attach $filefd
	         $tab(emod) eventtrace $bt
              }
              unset tab
       }
}


Error instproc pkt_list {todo pkts} {
        $self instvar serial_
                
	foreach item $serial_ {
                array set tab $item
	        if {$tab(type) == "List"} {
                   if {$tab(todo) == $todo} {
	    	      #puts "emod: [$emod info class]"               
                      puts "pkt_list $todo $pkts"
		      $tab(emod) droplist $pkts
                   }
	        }
                unset tab
	}
}

Error instproc reorder-rv { rvtype paramlist } {
        $self instvar serial_

        foreach item $serial_ {
                array set tab $item
	        if {$tab(todo) == "delay"} {
	    	   #puts "emod: [$emod info class]"
                   $self add-rvdelay $tab(emod) \
                                     $rvtype \
                                     $paramlist
	        }
                unset tab
	}
}




#-----------------------------------------------------------------------
Classifier/Hash/Fid instproc unknown-flow { src dst fid } {
        $self instvar em_
        set slot [$em_ new-flow $src $dst $fid]
}


ErrorModule instproc init { cltype { clslots 29 } } {
        $self instvar classifier_ lfid_ ffid_

        $self next
        set nullagent [[Simulator instance] set nullAgent_]

        set classifier_ [new Classifier/Hash/$cltype $clslots]
        $self cmd classifier $classifier_
        $self cmd target [new Connector]
        $self cmd drop-target [new Connector]
        $classifier_ set em_ $self
        set lfid_ {}
        set ffid_ {}
}


ErrorModule instproc new-flow {src dst fid} {
        $self instvar classifier_ lslot_ ffid_ lfid_

        if {[llength $lslot_] == 0} {
                set ns [Simulator instance]
                puts-error "[format %8.5f [$ns now]] Missing slot in the pool for flow $fid in pattern xxx \nflow: \{$lfid_\} \n $ffid_"
        }
        # is it a new flow ?
        if {[lsearch -exact $ffid_ $fid] < 0 } {
                set slot [lindex $lslot_ 0]
                set lslot_ [lrange $lslot_ 1 end]
                # populate hash classifier
                if {[isdebug]} {
                        set lfid_ [concat $lfid_ $fid]
                }
                $classifier_ set-hash auto 0 0 $fid $slot
        } elseif {[isdebug]} {
                puts -nonewline "-->"
        }
        if {[isdebug]} {
                $self instvar pat
                set traffic [$pat set traffic_]
                puts "f $fid E New IP [$traffic inprogress?] S [llength $lslot_] \{$lfid_\}"
        }
}


ErrorModule instproc ending-flow {src dst fid} {
        $self instvar classifier_ lslot_  ffid_

        if {[catch "$classifier_ del-hash 0 0 $fid" slot]} {
                # no slot found
                return
        }
        set lslot_ [linsert $lslot_ end $slot]
        lappend ffid_ $fid 
        if {[isdebug]} {
                $self instvar pat lfid_
                set pos [lsearch -exact $lfid_ $fid]
                if {$pos <0} {
                        puts "doesn't find $fid in $lfid_"
                        exit 1
                }
                set lfid_ [ldel $lfid_ $pos ]
                set traffic [$pat set traffic_]
                puts "f $fid E End IP [$traffic inprogress?] S [llength $lslot_] \{$lfid_\}"
        }

}
