#
# Copyright (c) 2006 University of La Reunion - France
# Author:   Pascal ANELLI
# http://www.univ-reunion.fr/~panelli
# All rights reserved.
# 
# Permission to use and copy this software in source and binary forms
# is hereby granted, provided that the above copyright notice, this
# paragraph and the following disclaimer are retained in any copies
# of any part of this software and that the University of La Reunion is
# acknowledged in all documentation pertaining to any such copy
# or derivative work. The name of the University of La Reunion may not
# be used to endorse or promote products derived from this software
# without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
# EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTIES OF 
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL 
# THE UNIVERSITY OF LA REUNION BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
# SOFTWARE.
#
#

#puts "#load test-grapher.tcl"

Class Grapher 

Grapher instproc init {toplot datafile verbose} {
	$self instvar graphfilename_ suffixgrapher_
	$self instvar verbose_ inputfile_ toplot_ 

	set suffixgrapher_ [GetSuffix [file tail [$self info class]]]
	set verbose_ $verbose
        
	# set the graph file
	set graphfilename_ [file rootname $datafile]
	append graphfilename "." $suffixgrapher_

	# datafile to plot
	set inputfile_ $datafile
	# what to plot
	set toplot_ $toplot
}


Grapher instproc draw {quiet} {
	$self instvar toplot_ quiet_ verbose_
       
	set quiet_ $quiet
        if {$verbose_} {
	        puts "     Call plotting $toplot_"
	}
        if {![$self valid-file]} {
                return
        }
        # find out false metric
        if {[lsearch -exact  {log event} $toplot_] < 0 } {
                $self dolines
                if {[catch "eval $self $toplot_" msg]} {
                       #puts "call param"
	               $self param
	        }
        } elseif {$verbose_} {
                        puts "     Warning:  Datafile is  not in format to plot"
        }
}


Grapher instproc valid-file {} {
	$self instvar inputfile_
       
        file stat $inputfile_ tab
	if {$tab(size) == 0} {
		puts "\t $inputfile_: empty file"
		return 0
	}
        return 1
}


Grapher instproc param {} {
	$self instvar inputfile_ toplot_

	$self graph "$toplot_" "$toplot_"
}


Grapher instproc finishstate {inputfile} {

set awk_code {
BEGIN {
	newindex = 0
	lastline = ""
        lib=""
      }
NF == 0 {
	if (newindex) {
		print lib, lastline
		lastline = ""
                lib=""
		newindex =0
	}
	next
	}
/^\"/ || /^#/ {
	 newindex = 1
	 lib=substr($0,2,length($0)-2) 
         lastline="0        0"
         next
	}
NF >= 2 {
	lastline = $0
	}
END { print lib, lastline}
}
	#puts -nonewline "  Finishstate: "
	set laststate [exec awk $awk_code $inputfile]
	return $laststate
}

Grapher instproc cat {inputfile} {

	set filefd [open $inputfile r]
	while {[gets $filefd line]>= 0} {
		puts $line
    	}
	close $filefd
}


Grapher instproc OverAll {} {
	# dummy proc
}

Grapher instproc PerFlow {} {
	# dummy proc
}


#----------------------------------------------------
#                METRIC
#----------------------------------------------------
Grapher instproc length {} {
	$self instvar inputfile_ 
       
        $self graph  "Queue length " "Size (pkt)"
}

Grapher instproc inputcurve {} {
	$self instvar inputfile_

	$self graph "Input curve" packet
}

Grapher instproc delay {} {
	$self instvar inputfile_

	$self graph "Delay" delay
}


Grapher instproc dropratio {} {
	$self instvar inputfile_

	$self graph "Drop ratio (%)" "ratio (%)"
}


Grapher instproc dropput {} {
	$self instvar inputfile_

        $self graph "Drop put"  "rate (%)" 
}

Grapher instproc receivepkt {} {
	$self instvar inputfile_

	$self coutingT
	$self graph  "received packet" "packet"
}

Grapher instproc droppkt {} {
	$self instvar inputfile_

	$self coutingT
	$self graph  "Dropped packet" "packet"
}

Grapher instproc sendpkt {} {
	$self instvar inputfile_

	$self coutingT
	$self graph  "Sent packet" "packet"
}

Grapher instproc sendrate {} {
	$self instvar inputfile_

	$self graph "Sending rate" "rate (%)"
}

Grapher instproc throughput {} {
	$self instvar inputfile_

	$self graph "throughput" "rate (%)"
}

Grapher instproc goodput {} {
	$self instvar inputfile_

	$self graph "Goodput" "rate (%)"
}


Grapher instproc throughput_TSW {} {
	$self instvar inputfile_

	$self graph "TSW throughput" "rate (%)"
}

Grapher instproc countingT {} {
	$self instvar inputfile_
	
	puts "flow <nl> time #packet total size"
	puts "[$self finishstate $inputfile_]"
}

Grapher instproc throughputT {} {
	$self instvar inputfile_

	puts "proto\tfid\tnodeid\ttime\trate"
	puts "[$self finishstate $inputfile_]"
	#exec echo call graph >/dev/tty
        $self graph "Total throughput" "rate (%)"
}

Grapher instproc goodputT {} {
	$self instvar inputfile_

	puts "proto\tfid\tnodeid\ttime\tgoodput"
	puts "[$self finishstate $inputfile_]"	
	$self graph "Total goodput" "rate (%)"
}


Grapher instproc dropratioT {} {
	$self instvar inputfile_

        puts "proto\tfid\tnodeid\ttime\tratio(%)"
	puts "[$self finishstate $inputfile_]"	
 	$self graph  "Total Drop ratio (%)"  "ratio (%)"
}

Grapher instproc statistics {} {
	$self instvar inputfile_

	exec cat $inputfile_
}

Grapher instproc dropprecedenceT {} {
	$self instvar inputfile_ 

        puts "proto\tfid\tnodeid\ttime\tpacket "
	puts "[$self finishstate $inputfile_]"	
	$self graph  "Drop precedence"  "packet"
}




#----------------------------------------------------
Class Grapher/xgraph -superclass Grapher

Grapher/xgraph instproc dolines {} {
}

Grapher/xgraph instproc exist-display {} {
        $self instvar quiet_
        global env

        if {[info exists env(DISPLAY)] && ! $quiet_} {
           return  1
        }
        return 0
}

Grapher/xgraph instproc graph {tittle ylib {xlib "time (s)"}} {
	$self instvar inputfile_ quiet_
       
        if {[$self exist-display]} {
                exec xgraph -bg white -bb -tk -x "$xlib" -y "$ylib" -t "$tittle" $inputfile_ &
        }
}


Grapher/xgraph instproc unknown {} {
	$self instvar inputfile_
	
        puts "  with file: $inputfile_ "
	puts "[$self finishstate $inputfile_]"	
	$self graph  "$inputfile_"  "Y"
}


Grapher/xgraph instproc activity {} {
	$self instvar inputfile_
       
        if {[$self exist-display]} {
                exec xgraph -bg white -bb -tk -nl -m -x time -y packets -t "activity" $inputfile_ &
        }
}

Grapher/xgraph instproc dropevent {} {
	$self instvar inputfile_
       
        if {[$self exist-display]} {
             exec xgraph -bg white -bb -tk -nl -m -x "time" -y "event" -t "Drop events" $inputfile_ &
        }
}

Grapher/xgraph instproc pdFunction {} {
	$self instvar inputfile_ quiet_
       
        if {[$self exist-display]} {
              exec xgraph  -bg white -bb -tk  -ly 0,1 -x value -y "prob" -t "pdf" $inputfile_ &
        }
}

Grapher/xgraph instproc cdFunction {} {
	$self instvar inputfile_ quiet_
       
        if {[$self exist-display]} {
              exec xgraph  -bg white -bb -tk -ly 0,1 -x value -y "prob" -t "cdf" $inputfile_ &
        }
}


#----------------------------------------------------
Class Grapher/gnuplot -superclass Grapher


# check if libs are in comment
Grapher/gnuplot instproc init {toplot datafile verbose} {
        $self instvar inputfile_ plotlib_ term_ outputfile_ 
        global conf opt

        $self next $toplot $datafile $verbose 

        set plotlib_ ""
        # find out terminal
        switch -exact $opt(t) {
                x { set conf(term) "set term x11" }
                a { set conf(term) "set term aqua" }
                e { set conf(term) "set terminal postscript enhanced color eps  'Helvetica' 14" }
                p { set conf(term)  "set terminal png small color" }
                default { set conf(term) $opt(t) }
        }
        
        set term_ $conf(term)
        regsub {\..*$} $datafile {} outputfile_
        
        set isindex [ exec sed -n {1 { 
                                        /#/p 
                                      } } $datafile | wc -l ]
        if {$isindex == 0 && [file size $datafile] > 0} {
                $self xtognu 
        } else {
                $self readindex
        }
}


Grapher/gnuplot instproc xtognu {} {
        $self instvar index_ inputfile_
        
set awk_code {
BEGIN {
        i=0 ;
      }
/^"/ {
	printf "%d %s\n", i, $0
        i+=1
     }
}

        set tmp [exec awk $awk_code $inputfile_]
        foreach i [split $tmp '\n'] {
                regsub -all {"} $i {} str ; # erase " char
                set item [split $str " " ]
                lappend index_ [list [lindex $item 0] [join [lrange $item 1 end]] ]
        }
        set tmpfile [exec mktemp /tmp/datafile.XXXX]
        exec echo "# $index_" >$tmpfile
        exec cat $inputfile_ | tr {"} {#} | sed -e {s/#$//} -e {1,3 {
                                                                     /^$/d
                                                                     } } >> $tmpfile
        if {[file size $tmpfile] > 0 } {
                exec mv -f $tmpfile $inputfile_
        }
}


Grapher/gnuplot instproc readindex {} {
        $self instvar index_ inputfile_
        
        set tmpst [exec head -1 $inputfile_ | sed {s/^#//} ]
        
        eval set index_ [list $tmpst]
}



Grapher/gnuplot instproc dolines {{what {($1):($2)}} {style lines}} {
        $self instvar index_ inputfile_

        foreach block $index_ {
                #set line "'$inputfile_' index [lindex $block 0] using $what with $style title \"[lindex $block 1]\""
                set line "'$inputfile_' index [lindex $block 0] using $what with $style title \"[lindex $block 1]\""
                $self addline $line
        }
}


Grapher/gnuplot instproc addline {line} {
        $self instvar plotlib_

        if { $plotlib_ == "" } {
                set plotlib_ $line
        } else {
               append plotlib_ ", " $line
        }
}


Grapher/gnuplot instproc graph {title ylib {xlib "time (s)"}}  {
        $self instvar toplot_ plotlib_ quiet_

                $self plot "$plotlib_"  "xlabel $xlib" "ylabel $ylib" "title $title"
}

Grapher/gnuplot instproc plot {stage args}  {
        $self instvar verbose_ quiet_ term_ inputfile_ outputfile_
        
        global conf opt
        
        set xlabel ""
        set ylabel ""
        set title "" 
        foreach i $args {
              eval set [lindex $i 0] "[lindex $i 1 ]"
        }
        
        set gplfile  ${outputfile_}.gpl
        
#set gnuplot "gnuplot -persist <<EOF
set gnuplot "
$term_
set grid ; 
set output '${outputfile_}.eps' ; 
set xlabel \"$xlabel\" ;
set ylabel \"$ylabel\" ;
set style data lines ;
set title \"$title\" ;
$conf(gnuplot)
plot $stage

"
#EOF"
        exec echo "$gnuplot" > $gplfile
        #exec cat $inputfile_ >> $gplfile
        if { ! $quiet_ } {
               if { $verbose_ } {
                        puts "\tgnuplot $opt(wd)/$gplfile"
               }
               exec gnuplot -persist $gplfile 
               exec  sleep 2 
        }
        #set tmpfile [exec mktemp /tmp/gnuplot.XXXX]
        #exec echo "$gnuplot" > $tmpfile
        #exec bash $tmpfile & 
        #exec  rm -f $tmpfile
}


