#
# Copyright (c) 2008 University of La Reunion - France
# Author:   Pascal ANELLI
# http://www.univ-reunion.fr/~panelli
# All rights reserved.
# 
# Permission to use and copy this software in source and binary forms
# is hereby granted, provided that the above copyright notice, this
# paragraph and the following disclaimer are retained in any copies
# of any part of this software and that the University of La Reunion is
# acknowledged in all documentation pertaining to any such copy
# or derivative work. The name of the University of La Reunion may not
# be used to endorse or promote products derived from this software
# without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
# EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTIES OF 
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL 
# THE UNIVERSITY OF LA REUNION BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
# SOFTWARE.
#
#


Class Math

Math instproc init {inputfile outputfile colx coly args} {
        $self instvar inputfile_ outputfile_ suffixdata_
        $self instvar colx_ coly_ AWK_
        
        set AWK_ "awk"

        set inputfile_ $inputfile
        set outputfile_ $outputfile
        set colx_ $colx
        set coly_ $coly
        if {[llength $args] > 0} {
                instance-parameter $self [lindex $args 0]
        }
        set suffixdata_  [GetSuffix data]
}


Math instproc mean {} {
       $self instvar inputfile_ AWK_ outputfile_
       $self instvar colx_ coly_ suffixdata_
                        
set awk_code {
/^\"/ || /#/  {
        print $0
        sum = 0.0
        nbrecord = 0
        next
        }
NF == 0 {
        printf "\n"
       }
$coly ~ /[0-9.\-]/{
        nbrecord++
        sum += $coly
        x= $colx
        print x, sum/nbrecord
       }

}
        exec $AWK_ $awk_code  colx=$colx_ coly=$coly_ $inputfile_ >>$outputfile_
}


Math instproc wmean {} {
       $self instvar inputfile_ AWK_ outputfile_
       $self instvar colx_ coly_ suffixdata_ weight_

set awk_code {
function last () {
        if (prev != -1) {
               print prev, sum/sumw, sumw, sum
               prev= -1
        }
}
BEGIN {
        prev =-1
      }
/#/ || /^$/ { last()
              print $0
              next
            }
NF >= 3   {
             if (prev == $colx) {
                sum += $coly
                sumw += $weight
             } else {
                if (prev == -1) {
                        prev= $colx
                        sum = $coly
                        sumw = $weight
                } else {
                        print prev, sum/sumw, sumw, sum
                        prev =$colx
                        sum = $coly
                        sumw = $weight
                }
            }
           }
END {
        last()
    }
}

       exec sort -n -k $colx_ $inputfile_ | $AWK_ $awk_code colx=$colx_ coly=$coly_ weight=$weight_ >>$outputfile_

}


Math instproc fmean {} {
       $self instvar inputfile_ AWK_ outputfile_
       $self instvar colx_ coly_ suffixdata_ maxslot_
                        
set awk_code {
function init () {
        sum = 0.0
        nbrecord = 0
        for (i=0; i<= maxslot ; i++) {
                val[i]=0.0
        }
}
BEGIN {
	init()
      }
\$1 ~ /#/ { next }
/^\"/ 	{
          init()
          print $0
        }
NF == 0 {
          printf "\n"
        }
$coly ~ /[0-9.\-]/ {
        i=nbrecord%maxslot
        sum-=val[i]
        val[i]=$coly
        sum += $coly
        x= $colx
        print x, sum/maxslot
        nbrecord++
       }
}
        exec $AWK_ $awk_code  maxslot=$maxslot_  colx=$colx_ coly=$coly_ $inputfile_ >>$outputfile_
}


Math instproc EWMA {} {
        $self instvar inputfile_ AWK_ outputfile_
        $self instvar colx_ coly_ suffixdata_ weight_
                
set awk_code {
        BEGIN {
                average =-1
        }
/^\"/ || /#/ {
        print $0
        average =-1
        next
      }
NF ==0 {
        printf "\n"
       }
NF >0 {
                if (average == -1) {average=$coly}
                 average *= (1-alpha)
                 average += $coly*alpha;
                 print $colx, average
       }
    END {
        }
}
        exec $AWK_ $awk_code  alpha=[expr 1.0*$weight_] colx=$colx_ coly=$coly_ $inputfile_ >>$outputfile_
}


Math instproc statistics {} {
	$self instvar inputfile_ AWK_ outputfile_
	$self instvar colx_ coly_ suffixdata_ 
	
set awk_code {
function printfinal() {
	if (nbSample != 0) {
		mean=sum / nbSample
		variance = (sumsquare/ nbSample) - mean*mean
		stdev = sqrt(variance)
	} else {
		mean=0
		stdev =0
		variance =0
	}
	print "   Samples: " nbSample, " Mean: ", mean, " Stddev: ", stdev, " min: ", min, " Max: ", max
}
BEGIN {
	sum =-1;
}
/^\"/ || /#/ {
		if (sum != -1)  { printfinal() }
	     	print "Statistic fid", $coly
			sum = 0.0
			sumsquare = 0.0
			nbSample = 0
			max=0
			min=-1		
		next;
	}
NF == 2 {  
		sum +=  $coly
		sumsquare += $coly*$coly
		nbSample++
		if ($coly > max)
				max=$coly
		if (($coly < min) || (min == -1))
				min= $coly
      }
END {
	printfinal()
    }
}

    exec $AWK_ $awk_code colx=$colx_ coly=$coly_ $inputfile_ >>$outputfile_	
}



Math instproc pdf {binsize} {
	$self instvar inputfile_ AWK_ outputfile_
        $self instvar colx_ coly_ suffixdata_ 

# prepare the data according to precision
set awk_accurate {
        BEGIN {
                nbligne = 0
              }
NF == 0 || /^\"/ || /^#/ {
        next
	}
{
	bin = int($col/binsize)
	printf "%d %-8.5f\n", bin, bin*binsize
        nbligne++
}
END {}
}
# compute the pdf
set awk_density {
BEGIN {
	slotvalue=-1; numbervalue= 0.0
     }
     {
       if (slotvalue == $1) 
       		numbervalue++
	else {
		if (slotvalue != -1) { 
                        printf "%8.5f\t%8.5f\n", slotvalue, numbervalue/nbrecord;
                }
		slotvalue=   $1
		numbervalue= 1.0
	}
     }
 END {
        printf "%8.5f\t%8.5f\t%d\n", slotvalue, numbervalue/nbrecord, numbervalue;
	printf "\n\n"
     }
}
#------
        puts stderr "On file: $inputfile_"
        exec $AWK_ {$0~/[#"']/ {print $0}}  $inputfile_ >> $outputfile_
	#"
        set tmp [makefile  "tmpint" $suffixdata_]
        exec $AWK_ $awk_accurate binsize=[expr 1.0*$binsize]  col=$coly_ $inputfile_ >>$tmp
	#	find out the number of line
        set pat  {{^[ 0-9.+-]+$}}
        set cmdline "grep -E $pat $tmp | wc -l"
        #puts "$cmdline"
        if {![catch "exec $cmdline" nbline]} {
           #puts "     found nbline: $nbline"
	   exec sort -n -k 1 $tmp |  $AWK_ $awk_density nbrecord=$nbline  >>$outputfile_
         } else {
                exec echo "0  1"  >> $outputfile_
                exec echo "" >> $outputfile_
                exec echo "" >>$outputfile_
         }
         exec rm $tmp
}


Math instproc cdf {} {
        $self instvar inputfile_ AWK_ outputfile_
        $self instvar colx_ coly_ suffixdata_

set awk_code {
        BEGIN {
                cumul=0.0
              }
             {
               cumul++
               print $1, cumul/nbrecord
             }
        END {
              printf "\n\n"
        }
}
        set tmp [makefile  "tmpcdf" $suffixdata_]
        exec $AWK_ {$0~/[#"']/ {print $0; exit}}  $inputfile_ >> $outputfile_
        exec $AWK_ {$0!~/[#"']/ && $0!~/^$/ {print $field}} field=$coly_  $inputfile_ | sort -n >> $tmp
        #
        #puts " $coly_ $tmp"
        set pat  {{^[ 0-9.+-]+$}}
        set cmdline "grep -E $pat $tmp | wc -l"
        #puts "$cmdline"
        if {![catch "exec $cmdline" nbline]} {
                exec $AWK_ $awk_code nbrecord=$nbline $tmp >>$outputfile_
        } else {
                exec echo "0 0 \n\n"  >> $outputfile_
        }
        #exec rm $tmp
}
