#
# Copyright (c) 2006 University of La Reunion - France
# Authors:   Pascal ANELLI
#            Fanilo HARIVELO
# http://personnel.univ-reunion.fr/panelli
# All rights reserved.
# 
# Permission to use and copy this software in source and binary forms
# is hereby granted, provided that the above copyright notice, this
# paragraph and the following disclaimer are retained in any copies
# of any part of this software and that the University of La Reunion is
# acknowledged in all documentation pertaining to any such copy
# or derivative work. The name of the University of La Reunion may not
# be used to endorse or promote products derived from this software
# without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
# EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTIES OF 
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL 
# THE UNIVERSITY OF LA REUNION BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
# SOFTWARE.
#
#

#puts "================================================ Post Suite"

#puts "load test-post-template.tcl"
source test-metric.tcl
source test-grapher.tcl

#========================================
Class PostSuite
#========================================
PostSuite instproc init {test verbose grapher conf } {
	$self instvar conf_ test_ verbose_
	$self instvar plotid_ 
	$self instvar grapherid_ 
	$self instvar plot_ 
	$self instvar grapher_ suffixdata_

        set plotid_ 0
	set grapherid_ 0
	set verbose_ $verbose
	set suffixdata_ [GetSuffix data]
	set test_ $test
	# Post process configuration 
        # array set conf_ [array get conf]
	array set conf_ $conf

	set grapher_ $grapher
	set plot_ $conf_(plot)

	# Process the asking analysis (post process option)	
	$self SetMetric
}


# Process the tracefiles in order to get data to plot
PostSuite instproc ComputeData {quiet} {
	$self instvar post_ verbose_

        set msg ""
        foreach i [array names post_ ] {
                if {$verbose_} {
	        	exec echo  "Call post id $i" >/dev/tty
		}
                set ldatafile [$post_($i) MakeData]
		if {[llength $ldatafile] >0} {
		       set newdata [$self mergedata $ldatafile] 
		       if {$newdata != $ldatafile} {
			   $post_($i) set log_ $newdata
                           if {$verbose_}  { 
                             exec echo  "     Merged datafile $newdata" >/dev/tty
		           }
		       }
		       foreach i $newdata {
                                set metric [lindex $i 0]
                                set datafile [lindex $i 1]
                                append msg $metric " " $datafile "\n"
                       }
        	}
	}
        if {$verbose_} {
	        exec echo "   ---- " >/dev/tty
        }
        return "$msg"
}


# Draw graph according to data file
PostSuite instproc DrawGraph {quiet} {
	$self instvar verbose_ post_
	$self instvar graph_ suffixdata_ grapher_
	$self instvar grapherid_ verbose_

	foreach i [array names post_ ] {
                if {$verbose_} {
	        	exec echo  "Call post id $i" >/dev/tty
		}
                set ldatafile [$post_($i) getldatafile]
		if {[llength $ldatafile] == 0} {
			continue
		}
		# create a grapher per metric 
		foreach j $ldatafile {
                   set metric [lindex $j 0]
		   set datafile [lindex $j 1]
                   if { $datafile != "" && [file size $datafile] > 0} {
                   	if {$verbose_} {
			   puts "     Create Grapher $grapherid_ $grapher_ for $metric $datafile"
		        }
		  	set graph_ [new Grapher/$grapher_ $metric $datafile $verbose_ ] 
                   } else {
                        if {$verbose_} {
                           puts "     WARNING: no graph for $metric"
			   continue
                        }
                  }
	         $graph_ draw $quiet
                 incr grapherid_
		}
	}       
        if {$verbose_} {
                exec echo "   ---- " >/dev/tty
        }
}


PostSuite instproc mergedata {ldatafile} {
	$self instvar graph_ plot_ suffixdata_ grapher_
	$self instvar grapherid_ verbose_ 
	
	# Merge files according plot_
	foreach operation $plot_ {
		set operationname [lindex $operation 0]
                set index -1
                set i 0
                foreach item $ldatafile {
	                set code [lsearch -exact $item $operationname]
                        if {$code >=0} {
                                set index $i
                                break
                        }
                        incr i
                }
		#puts "CreatePlot:  $operationname index: $index"
                if {$index == "-1" } {
			#puts "ERROR in CreatePlot: $operationname not found"
			continue
		}
		if {"[lindex [lindex $ldatafile $index] 0]" != "$operationname" } {
		      continue
		}
		set operationfile [lindex [lindex $ldatafile $index] 1]
		set resultfile [file rootname $operationfile]
		append resultfile "comp." $suffixdata_
		if {$verbose_} {
                        puts -nonewline "     plot: $operationname in "
		        puts "$resultfile"
                }
		catch "exec rm -f $resultfile"
		# Copy the base file in resultfile
		exec cat $operationfile > $resultfile
		set ldatafile [lreplace $ldatafile $index $index]
		foreach merged [lrange $operation 1 end] {
			set index [lsearch -regexp $ldatafile $merged]
			if {$index >=0} {
				set mergedfile [lindex [lindex $ldatafile $index] 1]
				exec cat $mergedfile >>$resultfile
				set ldatafile [lreplace $ldatafile $index $index]
			}
		}
		lappend ldatafile [list $operationname $resultfile]
	}
	return  $ldatafile
}



PostSuite instproc SetMetric {} {
    $self instvar post_ test_ conf_ plotid_ verbose_


	foreach scope $conf_(pkt) {
		set scopename [lindex $scope 0]
		set scopelist [lindex $scope 1]
		
	        if {[llength $scopelist] == 0} {
			continue
		}
	       foreach item $scopelist {
                        if {$verbose_} {
		   	  puts "Create post id $plotid_ for $scopename with {$item}"
			}
                        set post_($plotid_) [new Metric/$scopename $test_ $conf_(parameter) $item $verbose_]
	         	incr plotid_
	  	}
	}
	
	# tracedvar list in the format : scope { {who what}}
	foreach scope $conf_(param) {
		set tracedobject [lindex $scope 0]
		#puts "*** $scope"
                set scopelist [lindex $scope 1]
		#puts "**scopeitem $ltvar"
	        if {[llength $scopelist] == 0 } {
			continue
		}
	                    
              # check if the metric class for the tracedobject exist
              set ml [subclasslist Metric]
              if {[lsearch -regexp $ml $tracedobject] == "-1"} {
                  if {$verbose_} {
                     puts "WARNING: Metric/$tracedobject doesn't exist"
                  }
                  continue
              }
              foreach scopeitem $scopelist {
	        if {$verbose_} {
	                puts "Create post id $plotid_ for $tracedobject with {$scopeitem}"
	        }
                set post_($plotid_) [new Metric/$tracedobject $test_ $conf_(parameter) $scopeitem $verbose_]
	        incr plotid_
             }
	}			
        if {$verbose_} {
	        exec echo "   ---- " >/dev/tty
        }
}


#====================================================
#              Procedure definition
#====================================================      

proc setup_pprocess {prefix quiet verbose cleanup} {
        global opt conf

        if {$verbose} {
                exec echo "BEGIN post process" >/dev/tty
        }
        # Analysis param tab
        set post_suite [new PostSuite $prefix $verbose $opt(grapher) [array get conf] ]
	
        # Post process trace file to make of data file
       set msg [$post_suite ComputeData $quiet]
        
        # plot data file
        $post_suite DrawGraph $quiet

        if {$cleanup} {
                eval exec rm -f [glob -nocomplain *tmp* ] [glob -nocomplain *all\[0-9\]* ]
         }
        if {$verbose} {
                exec echo "END post process" >/dev/tty
        }

        return "$msg"
}

