#
# Copyright (c) 2010 University of La Reunion - France
# Author:   Pascal ANELLI
# http://personnel.univ-reunion.fr/panelli
# All rights reserved.
# 
# Format for short flow in prm file
#     TIME START FID CLASS TID SRCID SIZE INTERA NODEIDX FIP
#     TIME FINISH FID CLASS TID SRCID SIZE INTERA LATENCY FIP trans-pkts transm-bytes retrans-bytes lastack_number
        

#puts "#load test-source.tcl"

source test-RV.tcl

Class FidCnter

FidCnter instproc init {} {
	$self instvar lastfid_ lfid_

	set lfid_ {} ; # list of reserved fid
	set lastfid_ 0
} 

# create counter for traffic class
FidCnter instproc flow-new {} {
        $self instvar lastfid_
        
        set fid $lastfid_
        incr lastfid_
        
        return $fid
}


FidCnter instproc resv {fid} {
	$self instvar lfid_ lastfid_

	lappend lfid_ $fid
	if { [expr $lastfid_ - $fid] <= 0} {
		set lastfid_ [expr $fid + 1]
	}
}

FidCnter instproc get-lastfid {} {
        $self instvar lastfid_

        return $lastfid_
}


# Traffic: a set of pattern
#           counter of active session for a traffic class
Class TrafficClass

TrafficClass instproc init {idclass} {
        $self instvar ns_ logfile_ testsuite_
        $self instvar class_ flowinprogress_ tid_ lastarrival_

        set ns_ [Simulator instance]
        set testsuite_ [TestSuite instance]
        set logfilefd_ [$testsuite_ set logfilefd_]

        set class_ $idclass
        set flowinprogress_ 0
        set tid_          0
        set lastarrival_ 0.0
}

TrafficClass instproc class? {} {
        $self instvar class_

        return $class_
}

TrafficClass instproc start {} {
        $self instvar ns_ flowinprogress_ tid_ lastarrival_
        $self instvar intera_

        set now [$ns_ now]
        if {$lastarrival_ == 0.0} {
                set intera_ 0.0
         } else {
                set intera_ [expr $now - $lastarrival_]
         }
        set lastarrival_ $now
        incr flowinprogress_
        incr tid_

        return [list $flowinprogress_ $intera_ $tid_]
}

TrafficClass instproc finish {} {
        $self instvar flowinprogress_

        set flowinprogress_ [ expr $flowinprogress_ -1]
        return $flowinprogress_
}


TrafficClass instproc inprogress? {} {
        $self instvar flowinprogress_
        
        return $flowinprogress_
}

#-----------------------------------------------------------------------
#                      P A T T E R N 
#-----------------------------------------------------------------------
# Pattern: a set of source-destination node with a same traffic pattern
Class SourcePattern


SourcePattern instproc init {testsuite pattern replication} {
	$self instvar ns_ edge_ testsuite_ edgeid_ replication_ 
	$self instvar  core_ debug_ connection_ logfilefd_ class_ traffic_
        $self instvar type_ nbnode_
        $self instvar errortype_ id_ lfid_ fidcnter_
        global id

	set debug_ [isdebug]
	set ns_    [Simulator instance]
	set testsuite_   $testsuite 
        set replication_ $replication 
        set logfilefd_   [$testsuite set logfilefd_]
        set errortype_ "no"
	set lfid_ {} ;   # list of flow in progress (used for debug only)
       
	set fidcnter_ [$testsuite_ get-fidcnt]
	
        set id_ $id
        incr id
 
        #puts "new pattern:\n $pattern"
	#set testsuite_ [TestSuite instance]

	# process pattern parameter
	instance-parameter $self $pattern

        if {![info exists class_]} {
                set class_ besteffort
        }
        set traffic_ [$testsuite get-traffic $class_]

	if {[info exists edgeid_]} {
		set edge_ [$testsuite get-edge $edgeid_]
	} else {
		set core_ [$testsuite set corenetwork_]
        }
        if {$nbnode_ <= 0 } {
                return
        }
	$testsuite_ verbose "    type: [$self info class] class $class_"
        # create nodes (source and destination) of the pattern
	$self create-node
	# Assign edge node of the pattern
	$self assign-xgressnode
        # Start the flows in the pattern
        if {[info exists connection_] && ($connection_ == "Short")} {
                #puts "-->>$connection_"
                $self ScheduleTransfer 
        } else {
                #puts "-->>Long"
                set connection_ "Long"
                $self StartTraffic
       }
}

SourcePattern instproc didinit {patnum} {
	$self instvar flows_ nbnode_ id_ nbflow_

        if { $patnum != [expr $id_] } {
                puts-error "didinit: PB on pattern id $patnum != $id_"
        }

        for {set i 0} {$i < $nbnode_} {incr i} {
            set nbflow  [value $i $nbflow_] 
	    for {set j 0} {$j < $nbflow} {incr j} {
                $flows_($i,$j) didinit $patnum $i $j
            }
        }
}


SourcePattern instproc willrun {patnum} {
	$self instvar flows_ nbnode_ nbflow_

        for {set i 0} {$i < $nbnode_} {incr i} {
            set nbflow  [value $i $nbflow_] 
	    for {set j 0} {$j < $nbflow} {incr j} {
                $flows_($i,$j) willrun $patnum $i $j
            }
        }
}


SourcePattern instproc  select-ca {} {
        $self instvar flows_ ns_ nbnode_ type_ nbflow_
       
        if {$type_ != "TCP"} {
                return
        }
        for {set i 0} {$i < $nbnode_} {incr i} {
                set nbflow [value $i $nbflow_]
                for {set j 0} {$j < $nbflow} {incr j} {
                        $flows_($i,$j) select-ca
                }
        }
}

# create source and destination nodes (according nbnode_) 
SourcePattern instproc create-node {} {
	$self instvar flows_ node_ sourceparam_ nbnode_ nbflow_ type_
	$self instvar  ns_ nodetype_ testsuite_ traffic_ fidcnter_

	for {set i 0} {$i < $nbnode_} {incr i} {
		$testsuite_ verbose "*** Create pair of nodes index $i $nodetype_"
		set node_($i,s) [$ns_ $nodetype_]
		set node_($i,d) [$ns_ node]     

		# create flows in each node
		set nbflow  [value $i $nbflow_] 
                $testsuite_ verbose "***                                from [$node_($i,s) id] to [$node_($i,d) id]  with $nbflow flows"

		for {set j 0} {$j < $nbflow} {incr j} {
                     set flows_($i,$j) [new Flow/$type_ $self $node_($i,s) $node_($i,d)  $sourceparam_ [$fidcnter_ flow-new] $i $traffic_]
			# i is the nodeidx
		}
	}
}


# indicate the (i,e)gress node. gress node is node to be connected to the network
# by default, the source-destination is (i,e)gress node 
SourcePattern instproc assign-xgressnode  {} {
	$self instvar node_ nbnode_

	for {set i 0} {$i < $nbnode_} { incr i} {
		set node_($i,igress) $node_($i,s)
		set node_($i,egress) $node_($i,d)
	}
}

# For dynamic flow
SourcePattern instproc ScheduleTransfer {} {
        $self instvar RVsize_ RVdelay_ nbnode_ nbflow_ flows_ transfersize_
        $self instvar starttime_ idle_flow_ replication_ trafficseq_ fdseq_
        
        set RVsize_ [new RV $transfersize_ $replication_]
        set RVdelay_ [new RV $starttime_ $replication_]

        for {set i 0} {$i <$nbnode_} {incr i} {
                set nbflow  [value $i $nbflow_]

                for {set j 0} {$j < $nbflow} {incr j} {
                        set pos [expr ($i+1)*$j]
                        set idle_flow_ [linsert $idle_flow_ $pos $flows_($i,$j)] 
			$flows_($i,$j) conf-short $RVdelay_ $RVsize_
                 }
                 $self StartNode
        }
	if {[info exists trafficseq_]} {
		set fdseq_ [open $trafficseq_ w ]
	}
}


SourcePattern instproc StartNode {} {        
        $self instvar idle_flow_ ns_ nbnode_ id_ traffic_ nbflow_
        $self instvar testsuite_

proc   flowfid {idleflow} {
        set ff {}
        foreach flow  $idleflow {
                lappend ff "[$flow set fid_]"
        }
        return $ff
}        
        # Have we still available flow ?
        if {[llength $idle_flow_] == 0} {
                #set idle_flow_ [linsert $idle_flow_ end [$self create_flow] ]
                puts-error "[format %8.5f [$ns_ now]] Missing flows in the pool for pattern $id_ when InProg [$traffic_ inprogress?]"
        }
        # Take a flow at the beginning of the list
        set flow [lindex $idle_flow_ 0]
        set idle_flow_ [lrange $idle_flow_ 1 end]

         if {[isdebug]} {
		set fid [$flow set fid_]
	        set nodeidx [$flow set nodeidx_]
                puts "$id_ StartNode [format %-8.5f [$ns_ now]] NIDX $nodeidx fid $fid IF [llength $idle_flow_] InProg [$traffic_ inprogress?]"
        }
        $flow reschedule
}



SourcePattern instproc FinishTransfer {flow} {
        $self instvar ns_ testsuite_ idle_flow_ traffic_ id_ lfid_ nbflow_
      
        # Add flow at the end of the list
        set idle_flow_ [linsert $idle_flow_ end $flow]
        set inprogress [$traffic_ finish]
        
        if {[isdebug]} {
                set fid [$flow set fid_]
		set nodeidx [$flow set nodeidx_]
                set pos [lsearch -exact $lfid_ $fid]
                if {$pos <0} {
                        puts "FinishTransfer doesn't find $fid in $lfid_"
                        exit 1
                }
                set lfid_ [ldel $lfid_ $pos ]
                puts "$id_ FTransfer [format %-8.5f [$ns_ now]] NIDX $nodeidx fid $fid IF [llength $idle_flow_] IP $inprogress = \{$lfid_\}"
        }
}



# For Long Flow or  UDP
SourcePattern instproc StartTraffic {} {
        $self instvar starttime_ nbflow_ transfersize_ flows_ nbnode_ finishtime_
        
        for {set i 0} {$i <$nbnode_} {incr i} {
            set nbflow  [value $i $nbflow_]                
            set flowstart [value $i $starttime_]
            if {[info exists  transfersize_]} {
                set flowsize [value $i $transfersize_]
            } else {
                set flowsize -1
            }
	    if {[info exists finishtime_]} {
                set ftime [value $i $finishtime_]]
            } else {
                set ftime -1
            }
            for {set j 0} {$j < $nbflow} {incr j} {
               $flows_($i,$j) schedule-app [value $j $flowstart] [value $j $flowsize]
               $flows_($i,$j) stop-app  [value $j $ftime]
            }
        }
}

SourcePattern instproc print_start {flow nodeidx} {
        $self instvar traffic_ lfid_ id_ ns_
       
        set fid [$flow set fid_]
	set lfid_ [concat $lfid_ $fid]
        #set em [$flow set em_]; #  [llength [$em set lslot_]]
        puts  "$id_ STransfer [format %-8.5f [$ns_ now]] NIDX $nodeidx fid $fid = \{$lfid_\}"
}



#------------------------------------------------------------------------

Class SourcePattern/Classic -superclass SourcePattern

SourcePattern/Classic instproc init {testsuite pattern replication} {
	$self instvar nodetype_ idle_flow_
	
	set nodetype_ node
	set idle_flow_ {}
        
	$self next $testsuite $pattern	$replication
}

SourcePattern/Classic instproc print_start {flow nodeidx} {
        $self instvar traffic_ lfid_ id_ ns_
       
        set fid [$flow set fid_]
	set lfid_ [concat $lfid_ $fid]
        #set em [$flow set em_]; #  [llength [$em set lslot_]]
        puts  "$id_ STransfer [format %-8.5f [$ns_ now]] NIDX $nodeidx fid $fid IF [llength $idle_flow_] IP [$traffic_ inprogress?] = \{$lfid_\}"
}

#------------------------------------------------------------------------

Class SourcePattern/seq   -superclass SourcePattern

SourcePattern/seq instproc init {testsuite pattern replication} {
        $self instvar nodetype_ 

        set nodetype_ node

        $self next $testsuite $pattern  $replication
}

SourcePattern/seq instproc ScheduleTransfer {} {
        $self instvar nbnode_ nbflow_ flows_ idle_flow_ schedulist_ curf_ ns_ node_ trafficseq_

	# Read Sequence of flows
        set schedulist_ [$self readseq $trafficseq_]
        $self rsvfid

        for {set i 0} {$i <$nbnode_} {incr i} {
                 set nbflow  [value $i $nbflow_]
                 set idle_flow_($i) {}
                 for {set j 0} {$j < $nbflow} {incr j} {
                      lappend idle_flow_($i)  $flows_($i,$j)
		      # check nodeidx
		      if { [$node_($i,s) id] != [[$flows_($i,$j) set node_source_] id]} {
			  puts "ERROR in SourcePattern/seq: WRONG node assignation [$node_($i,s) id]"
			  exit 1
		      }
                 }
		#puts "idle_flow_($i) : $idle_flow_($i)" 
       }
      set curf_ 0 ;# current flow to launch
      $self startflows
}

SourcePattern/seq instproc rsvfid {} {
	$self instvar ns_ schedulist_ fidcnter_

	set ll [llength $schedulist_]
	for {set i 0} {$i < $ll } {incr i} {
			set fid [lindex [split [lindex $schedulist_ $i]] 5]
			$fidcnter_ resv $fid
	}
}

SourcePattern/seq instproc startflows {} {
      $self instvar ns_ schedulist_ curf_ idle_flow_
     
     set now [$ns_ now]

      while {[expr [llength $schedulist_] - $curf_] > 0 } {
                set line [split [lindex $schedulist_ $curf_]]
                #puts "Schedule seq: $line"
		set starttime [lindex $line 0]
                if { $starttime < $now} {
                        puts-error "[format %8.5f [$ns_ now]] Missing flows for $line"
                }
                set nodeidx [lindex $line 4]
                if { [llength $idle_flow_($nodeidx)] == 0 } {
                        break
                }
		# Take a flow at the beginning of the list
                set flow [lindex $idle_flow_($nodeidx) 0]
                set idle_flow_($nodeidx) [lrange $idle_flow_($nodeidx) 1 end] 
                
                set inter [lindex $line 1]
                set size [lindex $line 2]
		set fid [lindex $line 5]
		
		#puts "              on $starttime $inter $size $nodeidx $fid [[$flow set node_source_] id ]"
                $ns_ at $starttime "$flow start-app $inter $size $fid"
                incr curf_
        }
}

SourcePattern/seq instproc FinishTransfer {flow} {
        $self instvar ns_ testsuite_ idle_flow_ traffic_ id_ lfid_ nbflow_

      set nodeidx [$flow set nodeidx_]
      # Add flow at the end of the list
      set idle_flow_($nodeidx) [linsert $idle_flow_($nodeidx) end $flow]
      set inprogress [$traffic_ finish]
      $self startflows
}

SourcePattern/seq instproc print_start {flow nodeidx} {
        $self instvar traffic_ idle_flow_ lfid_ id_ ns_

        set fid [$flow set fid_]
        set lfid_ [concat $lfid_ $fid]
        #set em [$flow set em_]; #  [llength [$em set lslot_]]
        puts  "$id_ STransfer [format %-8.5f [$ns_ now]] NIDX $nodeidx fid $fid IF [llength $idle_flow_($nodeidx)] IP [$traffic_ inprogress?] = \{$lfid_\}"
}


# line format:
#     0          1     2     3           4       5
#   starttime, inter, size,  patternid, nodeidx, fid
SourcePattern/seq instproc readseq {file} {
        $self instvar testsuite_
        global opt

        $testsuite_ verbose "    read file  $opt(seq)/$file"
	set fd [open $opt(seq)/$file r]
        set condlist {}
        while {[gets $fd line] >=0} {
                lappend condlist $line
                }
        close $fd
        return $condlist
}


# line format:
#     0          1     2     3           4       5
#   starttime, inter, size,  patternid, nodeidx, fid
SourcePattern instproc writeseq {line} {
	$self instvar fdseq_
	if {[info exists fdseq_]} {
		puts $fdseq_ $line
	}
}


SourcePattern instproc closeseq {} {
	$self instvar fdseq_

	if {[info exists fdseq_]} {
		flush $fdseq_
		close $fdseq_ 
	}
}

#------------------------------------------------------------------------
#-------------------------------
#   ERROR Model
#-------------------------------

# add the error (a list of errors)  applied to all the flows of the node
SourcePattern instproc add-error-node {{returnpath 0} {front 1}} {
        $self instvar error_ replication_ nbnode_ flows_ testsuite_ hook_
        $self instvar errortype_ re_
       
        if {![info exists error_]} {
                puts "No Error Model node in pattern hook: $hook_ with $nbnode_ nodes"
                return
        }
        $testsuite_ verbose "*** Add Error on Node basis"
        set errortype_ "node"
        for {set i 0} { $i < $nbnode_} {incr i} { 
                if {$error_ == {}} {
                        continue
                }
                set ecn [[$flows_($i,0) set agent_] set ecn_]
                set em [new ErrorModule Fid 29]
                # insert error module in the link
                $self insert-errormodule $i $em $returnpath $front
                
                # create the serial of error model
                set re_($i,0) [new Error $error_ $replication_ [$em cmd drop-target] $ecn]
                # bind re to fid 0
                set head [$re_($i,0) get-head] 
                $em bind $head 0
                # default traffic passed to re also
                $em default $head
                # link rmod with the target of em
                $re_($i,0) add-target $em 
        }
}


# add the error module and create a pool of flow errors (a list of errors)
#  the flow error object is applied when a new flow is detected
SourcePattern instproc add-error-flow {{returnpath 0} {front 1}} {
        $self instvar error_ replication_ nbnode_ flows_ testsuite_ hook_
        $self instvar nbflow_ errortype_ re_

        if {![info exists error_]} {
                puts "No Error Model flow in pattern hook: $hook_ with $nbnode_ nodes"
                return
        }
        $testsuite_ verbose "*** Add Error on Flow basis"
        set errortype_ "flow"
        for {set i 0} { $i < $nbnode_} {incr i} {
                if {$error_ == {}} {
                        continue
                }
                # Add a err module for each node
                set em [new ErrorModule Fid $nbflow_]
                
                # XX to debug for callback
                $em set pat $self
                 
                $self insert-errormodule $i $em $returnpath $front
                set cls [$em cmd classifier]
                $cls defaulttarget [$em cmd target]

                # create the pool of slot (slot= entry of re in hashclassifier)
                # slot is a integer
                set lslot {}
                set ecn [[$flows_($i,0) set agent_] set ecn_]
                for {set j 0} { $j < $nbflow_} {incr j} {
                        set re_($i,$j) [new Error $error_ \
                                                  $replication_ \
                                                  [$em cmd drop-target] \
                                                  $ecn]
                        # link re with the target of em
                        $re_($i,$j) add-target $em
                        # install error in classifier
                        set slot [$cls installNext [$re_($i,$j) get-head] ]
                        lappend lslot $slot
                        
                        # For call back when flow is finished
                        $flows_($i,$j) set em_ $em 
                }
                # add the list of slot to em
                $em set lslot_ $lslot
        }
}

SourcePattern instproc get-error-flow0 {} {
	$self instvar error_ re_ errortype_ hook_ nbnode_

        if {![info exists error_]} {
                puts "No Error Model flow in pattern hook: $hook_ with $nbnode_ nodes"
                return
        }
        if { errortype_ != "flow" } {
		puts-error "WRONG Error model - not a flow based"
	}
	return re_(0,0)
}


# Add error module 
# in the acces or exit link (before the queue of link)
SourcePattern instproc insert-errormodule {index error returnpath front } {
        $self instvar node_ core_ ns_ hook_ testsuite_
        
        set nodes [$self localize-link $index $returnpath $front]
        set nodei [lindex $nodes 0]
        set nodee [lindex $nodes 1]
        $testsuite_ verbose "****Install error model ratio from node [$nodei id] to [$nodee id]"
        
        # call SimpleLink errormodule
        $ns_ lossmodel $error $nodei $nodee
        #  return [$ns_ link $nodei $nodee]
}   


SourcePattern instproc localize-link {index returnpath front} {
       $self instvar node_ core_ ns_ hook_
       
    set entrynode [lindex $hook_ 0]
    set exitnode  [lindex $hook_ 1]
    if {$front} {
        if {$returnpath} {
                set nodei  $node_($index,egress) 
                set nodee  [$core_ node? $exitnode]
        } else {
                set nodei  $node_($index,igress)
                set nodee  [$core_ node? $entrynode]
        }
     } else { 
        # Behind the corenetwork
        if {$returnpath} {
                set nodei [$core_ node? $entrynode]
                set nodee $node_($index,igress)
        } else {
                set nodei  [$core_ node? $exitnode]
                set nodee  $node_($index,egress)
        }
     }
     return [list $nodei $nodee]
}


# connect all node of the pattern to the network
SourcePattern instproc connect-pattern {queue} {
	$self instvar nbnode_ node_ edge_ bw_ tp_  core_ hook_ testsuite_

	for {set i 0} {$i < $nbnode_} {incr i} {
		#$testsuite_ verbose "*** Connect pair of nodes index $i"
		set linkbw [lvalue $i $bw_]
		set linktp [lvalue $i $tp_]
		
		if {[info exists edge_]} {
		   # give edge node of the pattern
	 	   $edge_ connect-flow  $node_($i,igress) $node_($i,egress) $linkbw $linktp $queue
		} elseif {[info exists core_]} {
		   $core_ connect-flow  $node_($i,igress) $node_($i,egress) $hook_ $linkbw $linktp $queue
		} else { puts-error "Where does pattern connect?"}
	}
}


SourcePattern instproc trace-error {todo paramfilefd} {
        $self instvar re_ nbnode_ errortype_ nbflow_
        
        if {![info exists re_]} {
                return
        }
        for {set i 0} { $i < $nbnode_} {incr i} { 
		$re_($i,0) tracevar $todo $paramfilefd
                
                if {[string equal $errortype_ "flow" ]} {
                   for {set j 1} {$j < $nbflow_} {incr j} {
                        $re_($i,$j) tracevar $todo $paramfilefd
                   }
                }
        }
}

#-------------------------
#         TRACING
#-------------------------

 # TCP PARAM 
        # enable source tracing
        #       enable TCP tracing, using to following some internal parameters 
        #
        # Use generic tracer
        #       set tracer [new Trace/Var]
        #       $tracer attach $pfd
        #       example:        $tcp(0) trace "ack_" $tracer
        # Or use the TCP tracer   like here     

#listvar format: { {{who} {what}}* }
#          who is the index or the index range of the object to trace
#              in the pattern. By default' who' is the node index in the pattern
#	   nota: Index start from 0
SourcePattern instproc tracevar {varlist on} {
	$self instvar testsuite_ node_ flows_ ns_ type_ nbflow_ id_

	   set paramfilefd [$testsuite_ set paramfilefd_]
	   if {[llength $varlist] !=2} {
		puts-error "in [info level [info level]]: Not enough element \n\t$varlist"
 	   }
           $testsuite_ verbose "***  	      Pattern id $id_"
	   set casting [lindex $varlist 0]
	   set tvars [lindex $varlist 1]
	   set lcasting [$self cast  $casting]
	   # Activate tracevar in Tracedobject 
	   foreach i $lcasting {
	    	        # is it an agent ?
		if {$on} {
                        set lflow [$self get-flows $i]
                        #puts "lflow $lflow for $i"
                        set lobject {}
                        foreach flow $lflow {
                                set lobject [concat $lobject [$flow get-agent]]
                        }
                        if {[$testsuite_ nam?]} {
                                foreach item $lobject {
                                   $item set nam_tracevar_ true
                                   $ns_ add-agent-trace $item [format "%s%02d" $type_ [$item set fid_]]
                                   $ns_ monitor-agent-trace $item
                                   foreach j $tvars {
                                        $testsuite_ verbose "[$item info class] $i trace ${j}_"
                                        $item tracevar ${j}_
                                   }
                                }
                                continue
                        }
		} else {
			# is it a box ?
			set lobject [$node_($i,igress) entry]
		}
		foreach item $lobject {
			$item attach $paramfilefd
			# loop on list of tracevar
			foreach j $tvars {
                         #	set tracer [new Trace/Var]
                         #      $tqracer attach  $paramfilefd
                         # Special case: event trace
                               if { ${j} == "event" } {
                                        $self event-trace $item $paramfilefd
                                } else {
                                        $testsuite_ verbose "[$item info class] trace ${j}_"
			                $item trace ${j}_
                               }
			}
		}
	   }
}

# Event tracing
SourcePattern instproc event-trace {object filefd} {

        set et [new BaseTrace/Event]
        $et attach $filefd
        $object cmd eventtrace $et
}


#-------------------------
#       Getter  
#-------------------------

SourcePattern instproc agent? {type} {
	$self instvar type_	

	return [string match $type $type_]
}


SourcePattern instproc contain? {type} {
	$self instvar nodetype_ node_ testsuite_

	if {$nodetype_ != "ConditionerNode"} {
		return 0
	}
        if {![info exists node_(0,igress)]} {
                $testsuite_ verbose "*** Empty pattern"
                return 0
        }
	set box [$node_(0,igress) entry]
	return [string match $type [$box info class]]
}

# return a list according to a simple regexp 
SourcePattern instproc cast {casting} {
	$self instvar nbnode_

	# Interpret the casting field
	set lcasting ""
	foreach element $casting {
		    if {$element == "*"} {
		        set lcasting ""   
		    	for {set i 0} {$i < $nbnode_} {incr i} {
				lappend lcasting $i
			}
		        break
		    }
		    if {[lsearch -regexp $element "-"] != -1} {
		    	set range [split $element "-" ]
			set max [lindex $range 1]
			if {$max == "\$"} { set max $nbnode_}
			for {set i [lindex $range 0]} {$i <= $max} {incr i} {
				lappend lcasting $i
			}
		     } else {
				lappend lcasting $element
		    }
	}
	return $lcasting
}

SourcePattern instproc get-node {nodeindex} {
        $self instvar node_ nbnode_

        if {$nodeindex >= $nbnode_} {
                puts-error  "get-node : node index $nodeindex over nbnode $nbnode_"
        }
        return [list $node_($nodeindex,s) $node_($nodeindex,d)]
}

# Return the list of flows in node nodeindex of pattern
SourcePattern instproc get-flows {nodeindex} {
        $self instvar nbflow_ flows_ nbnode_

      if {$nbnode_ == 0 } {
        return {}
      }
        
       set nbflow  [value $nodeindex $nbflow_]
       set lflow {}
       for {set j 0} {$j < $nbflow} {incr j} {
                lappend lflow $flows_($nodeindex,$j)
       }
       return $lflow
}


# return the value of the parameter
# The name of the parameter is given without trailing character i.e. "_"
SourcePattern instproc param-value {type paramname} {
	$self instvar type_
		
	if  {$type != $type_} {
		return ""
	}
	append paramname "_"
	return [$self set $paramname]
}


#TO DO
SourcePattern instproc debug {listdebug} {
}


SourcePattern instproc rttmin {} {
        $self instvar nbnode_ flows_ node_

        set lrtt {}
      # Compute RTT for each node of the pattern
        for {set i 0} {$i < $nbnode_} {incr i} {
  		set agent [$flows_($i,0)  get-agent]
  		set packetSize [$agent set packetSize_]
  		lappend lrtt [format "%-8.5f" [rtt $node_($i,s) $node_($i,d) $packetSize]]
	}
        return $lrtt
}


SourcePattern instproc print {fd} {
	$self instvar flows_ type_ hook_ edgeid_ node_ nbnode_ nbflow_ bw_ tp_
        $self instvar errortype_ re_ id_
        
	puts $fd "Source Pattern $id_: $type_"
	for {set i 0} {$i < $nbnode_} {incr i} {
		puts $fd "\t source node id: [$node_($i,s) id] index: $i"
		puts $fd "\t destin node id: [$node_($i,d) id] index: $i"
		if {[info exists edgeid_]} {
		puts $fd "\t	 connected to edge index $edgeid_"
		} else {
		puts $fd "\t     connected to network node id $hook_"
		}
                set tp [lvalue $i $tp_]
                set bw [lvalue $i $bw_]
                puts $fd "\t        access tp [value 0 $tp]  bw [value 0 $bw]"
                puts $fd "\t        exit   tp [value 1 $tp]  bw [value 1 $bw]"
  	        set nbflow  [value $i $nbflow_] 
	        for {set j 0} {$j < $nbflow} {incr j} {
		    $flows_($i,$j) print $fd $j
	        }
                if { [string compare $errortype_ "no"] != 0 } {
                        puts $fd "\t Error applied in pattern on $errortype_"
                        $re_($i,0) print $fd
                }
	}
}

#----------------------
#
#----------------------


# give (r,b) parameter for the token bucket policer
SourcePattern instproc ConfigurePolicer {} {
	$self instvar nbnode_ edge_ node_ cir_ cbs_ testsuite_
	
	set policy [value 0 [$edge_ set policy_]]
	if {$policy == "ANY"} {
		return
	}
	
	for {set i 0} {$i < $nbnode_} {incr i} {
		if { [isdebug] } {
			puts "Configure edge with source node: $i; value cir & cbs"
		}
		set flowcir [bw_parse [value $i $cir_]]
		set flowcbs [value $i $cbs_]
		
		$edge_ AddPolicy  $node_($i,s) $node_($i,d) $flowcir $flowcbs
	}
}


SourcePattern instproc add-tbf {shaper {returnpath 0} {front 1}} {
        $self instvar shaper_ nbnode_ ns_ policer_ testsuite_ 

        if { $shaper && ![info exists shaper_]} {
		puts "NO shaper defined"
                return
	}
        if { !$shaper && ![info exists policer_]} {
                puts "NO policer defined"
                return
        }
        
        if {$shaper} {
                set ltbf $shaper_
        } else {
                set ltbf $policer_
        }
        for {set i 0} {$i < $nbnode_} {incr i} {
           set prev {}
           for {set j 0} {$j < [llength $ltbf]} {incr j} {
                set param [value $j $ltbf]
                set tbf [new TBF]
                instance-parameter $tbf $param
                $tbf set rate_ [$ns_ bw_parse [$tbf set rate_]]

                set nodes [$self localize-link $i $returnpath $front]
                set nodei [lindex $nodes 0]
                set nodee [lindex $nodes 1]
                $testsuite_ verbose "Install tbf between node [$nodei id] - [$nodee id]"
                set link [$ns_ link $nodei $nodee]
                $tbf drop-target [$link set drophead_]
                if {$prev == {}} {
                        #$link add-to-head $tbf
                        set delay [$link set link_]
                        $tbf target [$delay target]
                        $delay target $tbf
                 } else {
                        $tbf target [$prev target]
                        $prev target $tbf
                 }
                 set prev $tbf
           }
        }
}




#====================================================
#                F L O W
#
# A pair of node: may have several flows
#=====================================================
Class Flow

Flow instproc init {pattern sourcenode destnode sourceparam fid nodeidx traffic} {
	$self instvar ns_ node_source_ node_dest_ traffic_ 
	$self instvar agentparam_ transfercnt_ nodeidx_ inter_ class_ fid_
        $self instvar pattern_ logfilefd_ testsuite_ tracefilefd_ fidcnter_

	set ns_          [Simulator instance]
	set node_source_ $sourcenode
	set node_dest_   $destnode
	set pattern_     $pattern
        set traffic_     $traffic
        set class_       [$traffic class?]
        set transfercnt_ 0; # number of object to transfer by this flow
        set nodeidx_     $nodeidx
        set logfilefd_   [$pattern_ set logfilefd_]
        set testsuite_   [TestSuite instance]
        set tracefilefd_ [$testsuite_ set paramfilefd_]
        set inter_       0.0
        set fid_         $fid
        set fidcnter_    [$testsuite_ get-fidcnt]
        
	instance-parameter $self $sourceparam

	if { [isdebug] } {
			puts "New flow : $fid_"
	}
	# Do you need different source per node in a pattern ?
	# $self new-flow $j $fid [value $nodeindex $agentparam_]
        $self new-flow
}

Flow instproc didinit {patnum nodenum flownum} {
        return
}


Flow instproc willrun {patnum nodenum flownum} {
        return
}

Flow instproc get-agent {} {
	$self instvar agent_
	return $agent_
}

Flow instproc print {fd index} {
	$self instvar agent_ class_
	puts $fd "\t\tagent class: [$agent_ info class] fid: [$agent_ set fid_] flow index: $index traffic class: $class_" 
}

Flow instproc unbind {fid} {
        $self instvar em_

        if {![info exists em_]} {
                return
        }
        $em_ ending-flow 0 0 $fid
}

#  Used by short and long flow /UDP
Flow instproc schedule-app {inter transfersize} {
	$self instvar ns_  fid_ flowstart_
        
        set flowstart_ [expr [$ns_ now] + $inter] 
	$ns_ at  $flowstart_  "$self start-app $inter $transfersize"
}


Flow instproc stop-app {stime} {
        $self instvar flowfinish_ app_ ns_
        
	set flowfinish_ $stime
        if {$stime >= 0} {
                $ns_ at $stime "$app_ stop"
        }


}

#--------------------------------------------------------

Class Flow/TCP -superclass Flow 

Flow/TCP instproc new-flow {} {
	$self instvar ns_ agent_ d_agent_ app_ trafficgenerator_
	$self instvar node_source_  node_dest_ agentparam_
	$self instvar tcpversion_ tcpsink_ fid_ packetSize_
	

        # create source and destination agent and connect them
        # Destination agent is a sink that will ack every packet

        # return a list of agent
        set agents [$ns_ create-connection-list $tcpversion_ $node_source_ $tcpsink_ $node_dest_ $fid_]
        set agent_ [lindex $agents 0] ; #source agent
        
        if {[info exists agentparam_]} {
	        instance-parameter $agent_ $agentparam_
        }
        
	set packetSize_ [$agent_ set packetSize_]
        # attach application object to the source agent
        if {![info exists trafficgenerator_]} {
                set trafficgenerator_ FTP
        }
        set app_ [$agent_ attach-app $trafficgenerator_]
        set d_agent_  [lindex $agents 1]
	if {[regexp "Full" $tcpsink_]} {
		 puts "FULL sink: do Listen"
                $d_agent_ listen
	}
        $agent_ set flow_ $self
}


Flow/TCP instproc select-ca {} {
        $self instvar agent_ tcpversion_ fid_ 
        $agent_ instvar congestion_avoidance_ ca_param_
       
        # definition in source param: congestion_avoidance_ ca_param_
        
        if {$tcpversion_ != "TCP/Linux"} {
                return
        }
        puts "select-ca $congestion_avoidance_ for flow $fid_"
        if {[info exists congestion_avoidance_]} {
                $agent_ select_ca $congestion_avoidance_
        }
        if {![info exists ca_param_]} {
                return
        }
        foreach param  $ca_param_ {
                # format ca_param_ {  {name value} ....}
                set name [lindex $param 0]
                if {[string compare $name "linux"] != 0} {
                   $agent_ set_ca_param $congestion_avoidance_ $name [lindex $param 1]
                   puts "TCP $congestion_avoidance_ set $name [$agent_ set_ca_param $congestion_avoidance_ $name]"
                } else {
                    foreach lparam  [lindex $param 1] {
                        # format linux_param_ {  {name value} ....}
                        $agent_ set_ca_param linux [lindex $lparam 0] [lindex $lparam 1]        
                   }
               }
       }
}

# For Dynamique flow only
Flow/TCP instproc conf-short {RVstart RVsize } {
        $self instvar RVstart_ RVsize_

        set RVstart_ $RVstart
        set RVsize_ $RVsize
}

Flow/TCP instproc reschedule {} {
        $self instvar testsuite_ RVstart_ RVsize_ nodeidx_ ns_
        $self instvar pattern_  transfersize_ packetSize_ fid_
	
        set now       [$ns_ now]
        if {$now <= 0} {
                set now 0.00001
        }
        set inter     [$RVstart_ value]
        set starttime [expr $now + $inter]
        if {$starttime <= $now} {
                puts "WARNING at $now: Flow/TCP reschedule at $starttime sec (not in the futur)"
        }
        set size [expr round([$RVsize_ value])]
	if {[isdebug]} {
		puts "          + [format %2.5f $inter]  sec = [format %8.5f $starttime]  $size bytes"
        }
	# Schedule the app starting  at inter second for here
        $self schedule-app $inter $size

        # Find a new flow and schedule it when the current flow starts
        $ns_ at $starttime "$pattern_ StartNode"
}


Flow/TCP instproc start-app {inter size {fid -1}} {
        $self instvar ns_ agent_  size_ inter_ logfilefd_ fid_ class_ app_ traffic_
        $self instvar node_source_  node_dest_ tracefilefd_ classintera_
        $self instvar pattern_ tid_ fidcnter_ transfercnt_ nodeidx_ flowstart_

        # Reinit the agents
	if {$fid == -1 } {
        	if {$fid_ == -1} {
                	set fid_ [$fidcnter_ flow-new]
                	$self reset-agent $fid_
		}
	} else {
		set fid_ $fid
                # use one fid from id space
		$self reset-agent $fid_
	}
        incr transfercnt_ 
        set flowstart_ [$ns_ now]
        set size_ $size
	set inter_ $inter
        $app_ send $size_

        set tmp [$traffic_ start]
        set fip        [lindex $tmp 0] ; # Flow in progress
        set classintera_ [lindex $tmp 1]
        set tid_         [lindex  $tmp 2]
        
        if {[isdebug]} {
                $pattern_ print_start $self $nodeidx_
                puts "[$pattern_ set id_] SFlow     [format %-8.5f [$ns_ now]] $inter_ $size_ $nodeidx_ $fid_ [$node_source_ id]"
        }
	# Log flow starting
        puts $logfilefd_ "[format %-8.5f [$ns_ now]] START  flow [format %3d $fid_] class $class_ from [format %3d [$node_source_ id]] to [format %3d [$node_dest_ id]] [format %6d $size_] bytes inter [format %.5f $classintera_] when [expr $fip-1] flows"

       puts $tracefilefd_ "[format %-8.5f [$ns_ now]] START $fid_ $class_ $tid_ [$node_source_ id] $size_ [format %.5f $classintera_] $nodeidx_  [expr $fip-1]"
	
	$pattern_ writeseq "[$ns_ now] $classintera_ $size_ [$pattern_ set id_] $nodeidx_ $fid_"
}


# The following procedure is called whenever a connection ends
Agent/TCP instproc done {} {
	$self instvar flow_
	$flow_ done [$self getstats]
}


#  return trans-pkts, transm-bytes, retrans-bytes, lastack number
Agent/TCP instproc getstats {} {
        $self instvar ndatapack_ ndatabytes_ nrexmitbytes_ ack_

        return [list $ndatapack_ $ndatabytes_  $nrexmitbytes_  $ack_ ]
}


# write stats in paramater file at the connection terminaison
Flow/TCP instproc done {lib} {
	$self instvar  pattern_ fid_  nodeidx_
	
        # record the ending flow
        $pattern_ FinishTransfer $self 
        # erase error condition for this flow (in ErrorModule)
        $self unbind $fid_
        # write statistics
        $self printstat $lib

        # Unassigned flow
        set fid_ -1
        # stop TCP agent
        $self reset-agent -1
}


Flow/TCP instproc printstat {lib} {
        $self instvar size_ agent_ flowstart_ ns_ classintera_ fid_ logfilefd_ tracefilefd_ pattern_ class_
        $self instvar node_source_  node_dest_ tid_ nodeidx_ traffic_

        set now [$ns_ now]
        set duration  [expr $now - $flowstart_]
        set fip [$traffic_ inprogress?]
        
        set tcpstat ""
        # lib is : ndatapack_ ndatabytes_  nrexmitbytes_  ack_
        foreach i $lib {
                append tcpstat [format "%d " $i]
        }
        
        puts $logfilefd_ "[format %-8.5f $now] FINISH flow [format %3d $fid_] \
                    class $class_ pattern [$pattern_ set id_] idx $nodeidx_ from [format %3d [$node_source_ id]] to [format %3d [$node_dest_ id]] ; \
                    inter [format %.5f $classintera_] when $fip flow ; duration [format %.5f $duration] sec [format %6d $size_] bytes TCPstat $tcpstat"
        
        puts $tracefilefd_ "[format %-8.5f $now] FINISH  $fid_ $class_ $tid_ [$node_source_ id] $size_ [format %.5f $classintera_] [format %.5f $duration] $fip $tcpstat"
}


Flow/TCP instproc reset-agent {fid} {
        $self instvar d_agent_ agent_

        $d_agent_ reset
        $agent_   reset

        $d_agent_ set fid_ $fid
        $agent_   set fid_ $fid
}


#==============================================================================

Class Flow/UDP -superclass Flow

Flow/UDP instproc new-flow {} {
	$self instvar ns_ agent_  fid_ pattern_
	$self instvar node_source_ node_dest_ agentparam_
        $self instvar app_ trafficgenerator_ trafficparam_
	
        # create source and destination agent and connect them
        # return a list of agent
        set agents [$ns_ create-connection-list UDP $node_source_ LossMonitor $node_dest_ $fid_ ]
        set agent_ [lindex $agents 0] ; #source agent
        $agent_ set d_agent_ [lindex $agents 1] ; #destination agent

        if {[info exists agentparam_]} {
	        instance-parameter $agent_ $agentparam_
        }

        set app_ [$agent_  attach-app $trafficgenerator_]
        if { $trafficgenerator_ == "Traffic/Exponential" || \
              $trafficgenerator_ == "Traffic/Pareto"} {
                set replication [$pattern_ set replication_]
                set randgen [new RNG]
                for {set i 1} {$i < $replication} {incr i} {
                              $randgen next-substream
                }
                $app_ use-rng $randgen
        }                                     
	instance-parameter $app_ $trafficparam_
 }



Flow/UDP instproc start-app {inter size {fid -1}} {
        $self instvar ns_ agent_  size_ logfilefd_ fid_ class_ app_ traffic_
        $self instvar node_source_  node_dest_ tracefilefd_ classintera_

        $app_ start
        set tmp [$traffic_ start]
        set fanum [lindex $tmp 0]
        set classintera_ [lindex $tmp 1]
        
        puts $logfilefd_ "[format %-8.5f [$ns_ now]] START  flow [format %3d $fid_] class $class_ from [format %3d [$node_source_ id]] to [format %3d [$node_dest_ id]] "

       puts $tracefilefd_ "[format %8.5f [$ns_ now]] START $fid_ $class_ [$node_source_ id]  [format %.5f $classintera_]"

}
