#
# Copyright (c) 2003 The University of La Reunion 
# Authors:  P. ANELLI,
#	    F. HARIVELO
#          http://personnel.univ-reunion.fr/panelli
# Contributed by the LIM Research Group, http://laboratoire.univ-reunion.fr/lim
#


#puts "#================================================ Test Suite"
#puts "#load test-suite-template.tcl"
source global.tcl
source template.rc
source test-topologies.tcl
source test-source.tcl
source test-error.tcl
source test-post-template.tcl

#====================================
##              Root Class         ##
#====================================

Class TestSuite

#---------------------------------------
Class Test/Default -superclass TestSuite

#=======================================================================

TestSuite set net_         BestEffort
TestSuite set patterntype_ Classic


TestSuite instproc init { args } {
	$self instvar ns_ net_ topo_ node_ patterntype_
	$self instvar test_ opt_ debug_ replication_ quiet_ verbose_
        $self instvar fidcnt_  traffic_ prefix_
        $self instvar suffixtrace_ suffixdata_ AWK_
        
	global opt
	
        # Default value for the modeling interface
        set AWK_ "awk"
        set suffixtrace_ [GetSuffix trace]
        set suffixdata_ [GetSuffix data]
	set fidcnt_ [new FidCnter]

        default-vars $self
        
        set quiet_ $opt(quiet)
        set verbose_ $opt(verbose)
        
        if {$opt(namgraph)} {
                set opt(nam) 1
         }
        
	array set opt_ [array get opt]
	# $self set-opt "[array get opt]"

	# create simulator instance
	set ns_ [new Simulator]
	if {[llength $args] > 0} {
	     if {[lindex $args 0] == "-multicast"} {
		$ns multicast
             }
	}
		
	# activate or not debug trace
	set debug_  [isdebug]
	
	# set others parameters
	if {![info exists test_]} {
		set name [$self info class]
        	set test_ [lindex [split $name /] 1]
	}
	
        set replication_ $opt(replication)
	global defaultRNG
	for {set i 1} {$i < $replication_} {incr i} {
              $defaultRNG next-substream
        }

        if {$opt(prefix) != "" } {
                set prefix_ $opt(prefix)
        } else {
                set prefix_ $test_
        }

	# Tune the simulation output
	$self SetOutput $opt(wd) $opt(nam) $prefix_
        

	# Set an event to stop the simulation and call the post process 
	if {$opt(stoptime) == 0} {
                puts-error "Simulation duration is ZERO sec"
        } 

        #------------------------
        #    MODEL INITIALISATION
        #------------------------
        $self init-model

        $self verbose "BEGIN didinit"
        $self didinit

}


TestSuite instproc init-model {} {
        $self instvar  opt_ pattern_
        $self instvar debug_
       global conf
                        
       # create topology class
        set corenetwork [$self create-topology]

        # configure queue size
        $corenetwork configure-corelink

        # Create traffic source
        $self create-pattern $opt_(pattern) $opt_(replication)

        # set specific measurments
        $self post-process $conf(param)

}


TestSuite instproc didinit {} {
        $self instvar pattern_ corenetwork_

        foreach i [array names pattern_] {
                  $pattern_($i) didinit $i
        }
        $corenetwork_ didinit
        $self verbose "END didinit"
}

TestSuite instproc run {} {
	$self instvar ns_ opt_ debug_
        $self instvar guide_
        
        if {[info exists guide_]} {
              puts "****\nTEST: $guide_\n****"
        }                               
        $self PrintTopology
        $self CommentAnimation
	
        $ns_ at 0.0 "$self willrun"
	$ns_ at $opt_(stoptime) "$self didrun"

	if { $debug_ } {
	        puts "DEBUG Section"
		parray opt_
	}
	$self verbose "END script file\n   ---- \nBEGIN simulation..."
	$self verbose "BEGIN willrun"
        $ns_ run
}


#Do something in run stage BUT before simulation has begun
TestSuite instproc willrun {} {
        $self instvar pattern_ corenetwork_

        foreach i [array names pattern_] {
                  $pattern_($i) willrun $i
                  $pattern_($i) select-ca
        }
        $corenetwork_ willrun
        $self verbose "END willrun"
        $self verbose "RUN ...."
}


TestSuite instproc didrun {} {
	$self instvar filenamenam_ filenamelog_
        $self instvar namout_  opt_ verbose_ prefix_ AWK_
        global env
        set status 0

        $self verbose "END Simulation\n   ---- \n# at [exec date]"
	# Close all the trace files
	$self closefd
        if {$verbose_ >= 2} {
                puts "LOG: \n[exec cat $filenamelog_ ]"
        }
	# Start network Animator
	if { $namout_ } {
               if {$opt_(namgraph)} {
                        if {$verbose_} { 
                                        puts "filtering..."
                                       }
                        set tmp [makefile tmp nam]
                        exec $AWK_ {$1 == "f" && $0~/null/ {next} {print $0}}  $filenamenam_ >> $tmp
                        exec mv -f $tmp $filenamenam_
                        exec tclsh $env(NAM)/bin/namfilter.tcl $filenamenam_
               }
  	       if {$verbose_} {
                        puts  "running nam..."
               }
	       exec nam $filenamenam_ &
	} elseif {$opt_(pp)} {
           #
	   # Call post process then drawing function
	   #
           set cmdline "ns $env(LIBSUITE)/post.tcl \
                                  -f $opt_(f) \
                                  -prefix $prefix_ \
                                  -wd $opt_(wd) "
           foreach i "quiet  verbose cleanup" {
                 append cmdline  "-" "$i " "\"$opt_($i)\" "
           }
           set tmpfile [exec mktemp /tmp/tmppost.XXXX]
	   if {$verbose_} {
                puts "$cmdline "
                set pid [exec tail -f $tmpfile & ]
           }
           if {[catch "eval exec $cmdline >> $tmpfile" msg]} {
              if {[lindex $::errorCode 0] eq "CHILDSTATUS"} {
                    set status [lindex $::errorCode 2]
                    puts "ERROR:  $::errorCode $msg"
              }
           }
           if {$verbose_} {
                exec kill -3 $pid 
           }
           exec rm -f $tmpfile
        }
        exit $status
}


TestSuite instproc SetOutput {wd nam prefix } {
	$self instvar ns_ namout_
        $self instvar filenamenam_    namfilefd_   suffixnam_ 
	$self instvar filenamepkt_    pktfilefd_   suffixpkt_
        $self instvar filenameparam_  paramfilefd_ suffixparam_ suffixdata_
        $self instvar filenamelog_    logfilefd_   suffixlog_ suffixtrace_
       
   global conf
       
	# Color assigned to FID for nam
	# Define different colors for data flows
	# format: color <fid> <color>
	$ns_ color 0 red
	$ns_ color 1 green
	$ns_ color 2 blue
	$ns_ color 3 yellow
	$ns_ color 4 purple
	$ns_ color 5 orange
	$ns_ color 6 pink
	$ns_ color 7 cyan

	# Find out suffix file name
	 set suffixpkt_ [GetSuffix pkt]
	 set suffixparam_ [GetSuffix param]
	 set suffixnam_ [GetSuffix nam]
         set suffixlog_ [GetSuffix log]
         set suffixtrace_ [GetSuffix trace]
         set suffixdata_ [GetSuffix data]

        
        # create a new working directory
	# remove all old trace files
	if {[file exists $wd] && [file isdirectory $wd]} {
		set rootdir [pwd] 
		cd $wd
		if {![catch "glob *.{$suffixpkt_,$suffixparam_,$suffixdata_,$suffixlog_,$suffixnam_} tmp*" todelete]} {
                        # 0= TCL_OK 
			eval exec rm -f $todelete
		}
		#puts [pwd]
		cd $rootdir 
		#exec rmdir $wd
	} else {
               exec mkdir $wd
	}
        #puts "[pwd] [file exists $wd]"
	# Change the working directory
	cd $wd

	# Set names for differents useful files
	$self set filenamepkt_ $prefix.$suffixpkt_
	$self set filenameparam_ $prefix.$suffixparam_
	$self set filenamenam_ $prefix.$suffixnam_
        $self set filenamelog_ $prefix.$suffixlog_
        
        set paramfilefd_ [open $filenameparam_ w]
	set logfilefd_ [open $filenamelog_ w]
        
	# activate trace for nam 
	set namout_ $nam
	if { $nam } {
	    set namfilefd_ [open $filenamenam_ w]
	    $ns_ namtrace-all $namfilefd_ 
	} elseif { $conf(pkt) != {} } { 
	     # Open the trace files
             set pktfilefd_ [open $filenamepkt_ w]

             # activate trace packets activity for all the link
	     $ns_ trace-all $pktfilefd_ 
      }
} 

TestSuite instproc nam? {} {
	$self instvar namout_
	
	return $namout_
}


TestSuite instproc paramfd? {} {
        $self instvar paramfilefd_

        return $paramfilefd_
}


TestSuite instproc closefd { } {
	$self instvar ns_ pktfilefd_ namfilefd_ paramfilefd_ logfilefd_ 	
	$self instvar pattern_ 

	
	$ns_ flush-trace
	foreach i [array names pattern_] {
		$pattern_($i) closeseq
		}
	if { [info exists pktfilefd_] } {
		close $pktfilefd_
		if { ![catch "$ns_ set WirelessNewTrace_"] && [$ns_ set WirelessNewTrace_] } {
			$self use-newtrace
		}
	}
             
        foreach item "namfilefd_ paramfilefd_ logfilefd_" {
	        if { [info exists $item] } {
		           eval close \$$item
	        }
        }
}


TestSuite instproc puts-log {msg} {
        $self instvar logfilefd_
        
        puts $logfilefd_ $msg
}

TestSuite instproc verbose {msg} {
        $self instvar verbose_

        if {$verbose_} {
                puts "$msg"
                $self puts-log "$msg"
        }
}


TestSuite proc instance {} {
	set lclass [subclasslist TestSuite]
	foreach test $lclass {
		set testsuite [$test info instances]
		if { $testsuite != "" } {
			return $testsuite
		}
	}
	puts-error "no TestSuite instance found"
}

#
#  To add comment in a nam animation
TestSuite instproc putcomment { ttime msg } {
 $ns_ at $ttime "$ns_ trace_annotate \"$msg\""
}

# Specialize this procedure for a specific simulation scenario (adding in rc file)
TestSuite instproc CommentAnimation {} {
}




#-----------------------------------------------#
#        TOPOLOGY
#-----------------------------------------------#

TestSuite instproc get-edge {edgeid} {
	$self instvar edge_
	
	return $edge_($edgeid)
}



TestSuite instproc create-topology {} {
	$self instvar opt_ ns_ edge_ corenetwork_ net_ debug_

	$self verbose "** Create core network"
	set corenetwork_ [new Core/$net_ $opt_(routelength) $opt_(corelink)]
	
     if {$net_ == "Diffserv"} {
	for {set i 0} { $i < [llength $opt_(edgelink)] } {incr i} {
		$self verbose "** create edge: $i"
		set edgelinkparam [value $i $opt_(edgelink)]
		set edge_($i) [new Edge/$net_ $corenetwork_ $edgelinkparam]
	}
     }
     return $corenetwork_
}

#
# to check the network topology 
TestSuite instproc PrintTopology {} {
	$self instvar corenetwork_ pattern_ logfilefd_

puts $logfilefd_ "-------------------x-------------------x--------------------"
puts $logfilefd_ " TOPOLOGY"
 $corenetwork_ print $logfilefd_
puts $logfilefd_ " TRAFFIC"
for {set i 0} {$i < [array size pattern_]} {incr i} {
 	$pattern_($i) print $logfilefd_
 }
puts $logfilefd_ "-------------------x-------------------x--------------------"
}


#-----------------------------------------------#
#        SOURCE
#-----------------------------------------------#

# patterntype_: Classic or DiffServ
TestSuite instproc create-pattern {pattern replication} {
	$self instvar ns_ pattern_ debug_ patterntype_ test_

	for {set i 0} { $i < [llength $pattern] } {incr i} { 
		$self verbose "** Create pattern: $i"

		# create pattern
	#	set code [catch {new SourcePattern/$patterntype_ $self [lindex $pattern $i]} pat]
	#	if {$code != 0 } {
		set pat [new SourcePattern/$patterntype_ $self [lindex $pattern $i] $replication]
	#	}
		set pattern_($i) $pat
		# connect sources and destination nodes of pattern to edge network
		#  or to the corenetwork 
		$pattern_($i) connect-pattern  DropTail
	}
}


TestSuite instproc get-pattern {type} {
	$self instvar pattern_ 

	set temp ""
	foreach i [array names pattern_] {
		if {$type == [$i set type_]} {
		      set temp [concat $temp $pattern_($i)]
		}
	}
	return $temp
}

# Return fid counter
TestSuite instproc get-fidcnt {} {
        $self instvar fidcnt_
        
        return $fidcnt_
}


TestSuite instproc get-traffic {id} {
        $self instvar traffic_

        if {![array exists traffic_]} {
                set traffic_($id) [new TrafficClass $id]
        } elseif {[llength [array names traffic_ $id]] == 0} {
                set traffic_($id) [new TrafficClass $id]
        }       
        return $traffic_($id)
}



#to tell in the script which packets are in error
# work ONLY for flow 0
TestSuite instproc  pkt_list {todo pkts} {
        $self instvar pattern_
                
         set re [$pattern_(0) get-error-flow0]
         $re pkt_list  $todo $pkts
}
                                
# return the source with fid

#----------------------------------------------------------#
#   POST PROCESS 
#----------------------------------------------------------#

TestSuite instproc post-process {paramlist} {
	$self instvar param_ 
	
	set param_ ""
	if { [llength $paramlist] == 0 } {
	                    return
	}
 
	set param_ $paramlist
        $self verbose "** Set tracing" 

        # for objects in pattern (as Agent or conditionner node)
        $self tracepattern 

        # For objects in Corenetwork as Queue
        $self tracequeue
}


TestSuite instproc tracepattern {} {
	$self instvar param_ pattern_
	
	if { [llength $param_] == 0 } {
		return
	}
	# process  parameters by object types
	foreach item  $param_ {
                $self verbose "***       trace var: $item"
		set typeobj [lindex $item 0]
		set totrace [lindex $item 1]
	
		# check if typeobj is an agent type then on= true
		set on [expr [lsearch -regexp [Agent info subclass] $typeobj] != -1]
                $self processtracevar $typeobj $totrace $on
	}
}


TestSuite instproc processtracevar {type ltodo where} {
	$self instvar pattern_
	
	#puts "Parameter scope $type \n\t\tTrace: $ltodo"
	foreach pat [array names pattern_] {
		if {$where} {
                         # on
                         # trace if agent is $type
			 if {![$pattern_($pat) agent? $type]} {
			   continue
	                 }
		} else { 
			# in
                        # trace if box is $type
			if {![$pattern_($pat) contain? $type]} {
				continue
			}
		}
		foreach item $ltodo {
			$pattern_($pat) tracevar $item $where
		}
	}
}


TestSuite instproc tracequeue {} {
	$self instvar corenetwork_ pattern_ param_  paramfilefd_

        set lq [subclasslist Queue]
	foreach scopeitem $param_ {
            # if {[lsearch -regexp $scopeitem "Queue"] != "-1"} {}
            set scope [lindex $scopeitem 0]
            # test if scope is a Queue class
	      if {[lsearch -regexp $lq $scope] != "-1" } {
	         $self verbose "***       trace var: $scopeitem"
                 $corenetwork_ tracequeue $scope [lindex $scopeitem 1] $paramfilefd_
            }
        }
}

 
# TO DO : Tgoodput
# No TCP flows:	
#		set goodput [expr [$opt_(destination) set bytes_]*8/[expr [$opt_(destination) set lastPktTime_] - $opt_(starttime)] ]
#	puts "Total recv thoughput from source: [$opt_(agent) set agent_addr_] fid  [$opt_(agent) set fid_] [format "%9.0f" $goodput ]bits/s\n"



TestSuite instproc Tgoodput {} {
	$self instvar ns_ 

	set now [$ns_ now]
	set lpattern [$self get-pattern TCP]
	
	foreach p $lpattern {
                               
		foreach a [$p get-agents TCP] {
		  set finishtime [$a set finistime_]
		  if {$finishtime == -1 } {
			set finistime $now
		  }
        	  set starttime [$a set starttime_]
		  set bytes [$a set ack_]
		
        	  set rate [expr $bytes*8*[$a set packetSize_]/($finishtime - $starttime)]

        	  puts  "tcp_goodput: agent [$a set agent_addr_ ] fid [$a set fid_] [format "%9.0f" $rate] bits/s $bytes"
		}
	}	
}


TestSuite instproc QueueMonitor {} {
	$self instvar corenetwork_ pattern_ param_  paramfilefd_

	# call  enable_traceQueue, monitor-flow
}


# Add a monitor on a specific link
TestSuite instproc enable_traceQueue {fdtracefile} {
	$self instvar opt_
	
	if {$fdtracefile == 0} {
		return
	}
	$opt_(ns) monitor-queue $opt_(n1) $opt_(n2) $fdtracefile
	$opt_(ns) at [$opt_(ns) now] "$opt_(bottleneck_link) start-tracing"
}


# 	x axis: time
#       y axis: queue size
#  Perform post-process from trace made by enable_traceQueue
TestSuite instproc length {} {
	$self instvar opt_
	
	# input format:
	set awk_queue {
		BEGIN {
		}
		/^\q/ {
			printf "%g %g\n", $3, $11;
		}
		END {
			printf "\n\n";
		}
	}


set awk_code {
        BEGIN{
	ave = 0.0;
	}
        /^\q/ { 
	  ave *= (1-q_weight);
	  ave +=  q_weight*$11;
          printf "%g %g\n", $3, $11;
        }
        END{ printf "\n\n"; }
}
#      exec awk $awk_code q_weight=$opt(q_weight) $test_.$opt(param) > $nameoutfile

	exec awk $awk_queue $tracefile_ 
}


#  perform post-process from trace made by enable_trace
# input format:
TestSuite instproc monitor-flow { fdtracefile sampleInterval } {
	$self instvar opt_
	
	if {$fdtracefile == 0} {
                return
        }
        set l01 [$opt_(ns) link $opt_(n0) $opt_(n1)]
	set fmon [$opt_(ns) makeflowmon Fid]
	$opt_(ns) attach-fmon $l01 $fmon
        $fmon attach $fdtracefile
	
	$opt_(ns) at $sampleInterval "$self DumpMonitor $fmon $fdtracefile $sampleInterval"
}




# Prints "time: $time class: $class bytes: $bytes" for the link.
TestSuite instproc DumpMonitor { fmon fdtracefile sampleInterval } {
	$self instvar opt_
	
	foreach flow [$fmon flows] {
		set bytes [$flow set bdepartures_]
		if {$bytes > 0} {
			puts $fdtracefile "fmon [$opt_(ns) now] x x bytes: $bytes $sampleInterval [$flow set flowid_]"  
			$flow reset
		}
        }
	#$fmon dump
        $opt_(ns) at [expr [$opt_(ns) now] + $sampleInterval] "$self DumpMonitor $fmon $fdtracefile $sampleInterval"
}



#----------------------------------------------------------------------
#           TRACING
#----------------------------------------------------------------------
TestSuite instproc use-newtrace {} {
	$self instvar opt_  namepktfile_

	
	set awk {
		function getValue(parameterName) {
			for(i=1;i<=NF;i++) {
				if($i == parameterName) {
					j = i + 1
					return $j
				}
			}
			return FFLAG
		}
		
		BEGIN {
			FEVENT=1 ; FTIME="-t"
			FINGRESS="-Hs" ; FEGRESS="-Hd"
			FPTYPE="-It" ; FPLEN="-Il"
			FFLAG="-------" ; FFID="-If"
			FSA="-Is" ; FDA="-Id" ;
			FSEQTCP="-Ps" ; FSEQACK="-Pa" ; FSEQUDP="-Pi" ; FUID="-Ii"
			FDROPREA="-Nw"
		}
		
		{
			event=$FEVENT
			if(event != "s" && event != "r" && event != "d" && event != "f")
				next       
			
			protocole=getValue(FPTYPE)
			if(protocole == "tcp")
				FSEQ=FSEQTCP
			else {
				if(protocole == "ack")
					FSEQ=FSEQACK
				else
					FSEQ=FSEQUDP
			}
			
			if(event=="s")
				event="+"
				
			if(event=="d" && getValue(FDROPREA) == "COL")
				event="c"
			
			if(event != "c" && (getValue(FTIME) == FFLAG || getValue(FINGRESS) == FFLAG || getValue(FEGRESS) == FFLAG || getValue(FPTYPE) == FFLAG || getValue(FPLEN) == FFLAG || getValue(FFID) == FFLAG || getValue(FSA) == FFLAG || getValue(FDA) == FFLAG || getValue(FSEQ) == FFLAG || getValue(FUID) == FFLAG))
				next
				
			#print event, getValue(FTIME), getValue(FINGRESS), getValue(FEGRESS), getValue(FPTYPE), getValue(FPLEN), FFLAG, getValue(FFID), getValue(FSA), getValue(FDA), getValue(FSEQ), getValue(FUID)
			# A ENLEVER, temporaire pour un reseau sans fils sans routage avec l'utilisation de DumpAgent
			fegress = getValue(FEGRESS)
			if(event=="r" && fegress < 0)
				fegress="0"
			print event, getValue(FTIME), getValue(FINGRESS), fegress, getValue(FPTYPE), getValue(FPLEN), FFLAG, getValue(FFID), getValue(FSA), getValue(FDA), getValue(FSEQ), getValue(FUID)
		}
		#END {
		#	split(args, tab, " ")          
		#	for (elt in tab)
		#	     print tab[elt]
		#}
	}
	
	exec $opt_(AWK) $awk $namepkfile > .$namepkfile
	exec mv $namepkfile $namepkfile.tmp
	exec mv .$namepkfile $namepkfile
}

