#!/bin/bash
# compute performance metric for each completed flow

# input file: *.prm
#   Time FINISH fid class nodeid size inter-arrival duration ndatapack ndatabytes  nrexmitbytes  ack
# output format
#     throughput

#================================================================
export LC_ALL=C #For sort -n problem

#====================================================
# Constantes
#====================================================

timetransit=100
bw=1000000
GNUPLOT_TERM="postscript eps enhanced color 'Helvetica' 12"
#   x11 or aqua

#====================================================
# Procedures Definition
#====================================================
function usage () 
{
      echo "Usage: sh $0 [-t TERMTYPE ] TRACEFILE [DATAFILE]"
      exit 1
}

#================================================================

#=======================================
##                 MAIN               ##
#=======================================

# Read option
while getopts t:T: option
do
       case $option in
           t|T)
                GNUPLOT_TERM=$OPTARG
		;;
	   \?)
                echo "Error: Unknown option"
                usage $*
                exit 1
                ;;
        esac
done
shift `expr $OPTIND - 1`

# Read TRACEFILE
if [ $# -lt 1 ]; then
        echo "ERROR: Wrong command line- No trace file given"
        usage
fi


prm=$1
shift
# Read the name of output datafile
name=${0%.*}
name=${name##*/}

if [ $# -eq 1 ]; then
	filename=$1
else
	filename=$name
fi	 

echo " *** FLOW  ***"

#=======================================
##                PLOT                ##
#=======================================

# for a graph F(flow)= throughput and F(rate) = Prob(R<rate)
############################################################
echo "# Make de cdf of throughput"
awkthru=/tmp/${name}thru.awk
cat >|$awkthru <<EOF
BEGIN {
	topbin=100
	for (i=0 ; i <= topbin ; i++) {
		bins[i]=0
	} 
	nbrecord=0
}
\$2 == "FINISH" {
		if (int(\$1) < timetransit) next
		thruput = (\$10*8.0/\$8)/($bw/100.0)
		printf "%7.2f\n", thruput > "tmp.data"
		cbin=int(thruput)
		if (cbin < 0) cbin = 0
		if (cbin > 100) cbin= 100
		++bins[cbin]
		++nbrecord
		}
END {
	sum=0.0
	for (bin=0; bin <= topbin ; bin++)  {
		prob= 1.0*bins[bin]/nbrecord
		printf ("%d %f\n", bin, prob) > "${filename}thrupdf.data"
		sum = sum + prob;
		print bin, sum;
	}
}
EOF

awk -f $awkthru timetransit=$timetransit $prm >|${filename}thrucdf.data

#   F(flow)= throughput
sort -n tmp.data >|${filename}thru.data

gnuplot -persist <<!
set term `echo $GNUPLOT_TERM`
set output "${filename}thru.eps"
set xlabel "Flow"
set ylabel "Throughput (%)"
unset key
unset title
set mytics 2
set yrange [0:70]
set xtics border nomirror
set ytics border nomirror
set border 3
plot '${filename}thru.data' using (\$1) with lines
!
# F(rate) = Prob(R<rate)
gnuplot -persist <<!
set term `echo $GNUPLOT_TERM`
set output "${filename}thrucdf.eps"
unset key
set xtics border nomirror
set ytics border nomirror
set border 3
set xlabel "Throughput (%)"
set ylabel "Fraction"
set title "Cumulatif density function"
set xrange [1:100]
set mxtics 5
plot '${filename}thrucdf.data' using (\$1):(\$2) with lines
!


# for a graph F(size)= Prob[S < size]
#####################################
#  Check the traffic model: many short flows and fews long flows 
echo "# Make de cdf of size"

tmpfile=/tmp/toprocess.data
awksize=/tmp/${name}size.awk
cat >|$awksize <<EOF
\$2 == "FINISH" {
		if (int(\$1) < timetransit) next
                # print size, thruput
		printf "%d %7.2f\n", \$10, (\$10*8.0/\$8)/($bw/100.0) 
		}
EOF

awk -f $awksize timetransit=$timetransit $prm | sort -n -k1  >| $tmpfile

echo " TO DO: plot the graph F(size)= Prob[S < size]"
# for a graph F(size)= Average throughput
########################################
echo " TO DO: plot the graph F(size)= Average throughput"
exit 0
