#
# Copyright (c) 2006-2009 University of La Reunion - France
# Author:   Pascal ANELLI
# http://personnel.univ-reunion.fr/panelli
# All rights reserved.
# 
# Permission to use and copy this software in source and binary forms
# is hereby granted, provided that the above copyright notice, this
# paragraph and the following disclaimer are retained in any copies
# of any part of this software and that the University of La Reunion is
# acknowledged in all documentation pertaining to any such copy
# or derivative work. The name of the University of La Reunion may not
# be used to endorse or promote products derived from this software
# without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
# EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTIES OF 
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL 
# THE UNIVERSITY OF LA REUNION BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
# SOFTWARE.
#
#

#  This test suite is for education labs
# to run an individual test :
# ns test-suite-edu.tcl <ressourcefile> [-testname name]
# ....
#
# To view a list of available tests to run with this script:
# ns test-suite-edu.tcl
#


#puts "   Read script file: [info script]"
#====================
# load common library
#====================
set currentdir [pwd]
catch "cd $env(LIBSUITE)"

source test-suite-template.tcl
catch "cd $currentdir"


#================================
# Parameter given by command line
#================================
# replication stoptime wd nam verbose quiet


#========================
# set default parameters
#========================

remove-all-packet-headers       ; # removes all except common
add-packet-header Flags IP TCP RTP  ; # hdrs reqd for TCP
#Trace set show_tcphdr_ 1

# Default parameters for TCP
Trace set show_tcphdr_ 0
Queue set limit_ 50
Agent/TCP set tcpTick_ 0.1
#The default for tcpTick_ 0.01
#  To be according to 2.4 (use a min rto 1 second in rfc 0.2 in ns)
Agent/TCP set rfc2988_ false
Agent/TCP set minrto_ 1
Agent/TCP set useHeaders_ false
# limited transmit
Agent/TCP set singledup_ 0
Agent/TCP set syn_ false
#  delay opening cwnd until 1st data recv'd : model SYN/ACK 
# model SYN and SYN/ACK
Agent/TCP set delay_growth_ false
Agent/TCP set windowInit_ 1
Agent/TCP set windowInitOption_ 1
   # 1= windowInit
   # 2= do iw according to rfc 3390
# true for unnecessary multiple Fast Retransmits: when multiple drop from a single window of data can sometimes result in multiple reductions of the congestion window
Agent/TCP set bugFix_ 1 

Agent/TCP set ecn_ 0


# don't allow the retransmit timer to expire for
# a window with many packet drops
Agent/TCP/Newreno set newreno_changes1_ 0

Agent/TCPSink/DelAck set interval_ 0.5
Agent/TCPSink/Sack1/DelAck set interval_ 0.5
Agent/TCP set maxburst_ 0
Agent/TCP/RFC793edu set syn_ false


set opt(stoptime) 20
set opt(verbose)  1 ;# 2 for 2 for print log file
set opt(nam) 0
set opt(quiet) 0


#=======================================
##           BASE CLASS            ##
#=======================================

Test/Default instproc didrun {} {
       

       $self next
        puts "\n>>>>>>>>>>>>>>>> Don't forget to read the logs  <<<<<<<<<<<<<<<<<<<<" 
        exit 0
}

#==========================================
##           OVERLOADED FUNCTIONS        ##
#==========================================

FQLink  instproc new-flow { src dst fid } {
        $self instvar classifier_ nactive_ queue_ link_ drpT_ 
        $self instvar queueManagement_ fq_

        incr nactive_ 

        set fq_ $queue_
        set limit_ [$fq_ set limit_]
        
        if {![info exists queueManagement_]} {
                set queueManagement_  [$class set queueManagement_]
        }
        set q [new Queue/$queueManagement_]
        $q set limit_ $limit_
        
        $q drop-target $drpT_

        set slot [$classifier_ installNext $q]
        $classifier_ set-hash auto $src $dst $fid $slot
        $q target $queue_
        $queue_ install $fid $q
}




#==========================================
##           SPECIALIZED CLASSES         ##
#==========================================

Class Test/TBF -superclass Test/Default

Test/TBF instproc create-pattern {pattern replication} {
	$self instvar pattern_ 
	
	$self next $pattern $replication

	for {set i 0} {$i <[llength $pattern]} {incr i} {
                # shaper to true (use field shaper of pattern) 
		$pattern_($i) add-tbf true 0 1
                # shaper to false = policer (use field policer of pattern)
                $pattern_($i) add-tbf false 0 0
	}
}


Class Test/ErrorAck -superclass Test/Default

Test/ErrorAck instproc create-pattern {pattern replication} {
	$self  instvar pattern_ 

	$self next $pattern $replication
	
	for {set i 0} {$i <[llength $pattern]} {incr i} {
		$pattern_($i) add-error-node 1 0
	}
}


Class Test/Error -superclass Test/Default

Test/Error instproc create-pattern {pattern replication} {
	$self  instvar pattern_ 

	$self next $pattern $replication
	
	for {set i 0} {$i <[llength $pattern]} {incr i} {
		$pattern_($i) add-error-node 0 0
	}
}


Class Test/ShortFlow -superclass Test/Error
# filter packet trace file: only finished flows are keeped

Test/ShortFlow instproc finish {} {
        $self instvar filenamepkt_ timetransit_ ns_
        global conf
        
        $self next
        # to init the timetransit
        instance-parameter $self $conf(parameter)
        
        set ifile "/tmp/$filenamepkt_"
        exec cp $filenamepkt_ $ifile
        set cmdline "completedflow.py -t \"$timetransit_\" <$ifile >$filenamepkt_"
        if {[file exists $filenamepkt_]} {
                        exec rm -f $filenamepkt_
        }
                 
        set errno [catch "exec $cmdline" msg]
        if {$errno == 1} {
                     puts "ERROR on result file:\n$msg\n"
                     exit 1
        }                           

}


Class Test/tahoe -superclass Test/Error

Test/tahoe instproc init {} {
        $self instvar tcpversion_ guide_

        set tcpversion_ TCP
        # TO DO Change tcpversion
        set guide_ "Tahoe with 1 spurious retransmit then 2 CE"
        Agent/TCP set singledup_ 1

        $self next
        $self pkt_list drop { 6 9 12 14}
}


#==============================
#            load Main
#==============================
source "$env(LIBSUITE)/test-main.tcl"

